# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

'''Strptime-related classes and functions.

CLASSES:
    LocaleTime -- Discovers and/or stores locale-specific time information
    TimeRE -- Creates regexes for pattern matching a string of text containing
                time information as is returned by time.strftime()

FUNCTIONS:
    _getlang -- Figure out what language is being used for the locale
    strptime -- Calculates the time struct represented by the passed-in string

Requires Python 2.2.1 or higher (mainly because of the use of property()).
Can be used in Python 2.2 if the following line is added:
    True = 1; False = 0
'''
import time
import locale
import calendar
from re import compile as re_compile
from re import IGNORECASE
from datetime import date as datetime_date
__author__ = 'Brett Cannon'
__email__ = 'brett@python.org'
__all__ = [
    'strptime']

def _getlang():
    return locale.getlocale(locale.LC_TIME)


class LocaleTime(object):
    '''Stores and handles locale-specific information related to time.

    This is not thread-safe!  Attributes are lazily calculated and no
    precaution is taken to check to see if the locale information has changed
    since the creation of the instance in use.

    ATTRIBUTES (all read-only after instance creation! Instance variables that
                store the values have mangled names):
        f_weekday -- full weekday names (7-item list)
        a_weekday -- abbreviated weekday names (7-item list)
        f_month -- full weekday names (14-item list; dummy value in [0], which
                    is added by code)
        a_month -- abbreviated weekday names (13-item list, dummy value in
                    [0], which is added by code)
        am_pm -- AM/PM representation (2-item list)
        LC_date_time -- format string for date/time representation (string)
        LC_date -- format string for date representation (string)
        LC_time -- format string for time representation (string)
        timezone -- daylight- and non-daylight-savings timezone representation
                    (3-item list; code tacks on blank item at end for
                    possible lack of timezone such as UTC)
        lang -- Language used by instance (string)
    '''
    
    def __init__(self, f_weekday = None, a_weekday = None, f_month = None, a_month = None, am_pm = None, LC_date_time = None, LC_time = None, LC_date = None, timezone = None, lang = None):
        '''Optionally set attributes with passed-in values.'''
        if f_weekday is None:
            self._LocaleTime__f_weekday = None
        elif len(f_weekday) == 7:
            self._LocaleTime__f_weekday = list(f_weekday)
        else:
            raise TypeError('full weekday names must be a 7-item sequence')
        if a_weekday is None:
            self._LocaleTime__a_weekday = None
        elif len(a_weekday) == 7:
            self._LocaleTime__a_weekday = list(a_weekday)
        else:
            raise TypeError('abbreviated weekday names must be a 7-item  sequence')
        if f_month is None:
            self._LocaleTime__f_month = None
        elif len(f_month) == 12:
            self._LocaleTime__f_month = self._LocaleTime__pad(f_month, True)
        else:
            raise TypeError('full month names must be a 12-item sequence')
        if a_month is None:
            self._LocaleTime__a_month = None
        elif len(a_month) == 12:
            self._LocaleTime__a_month = self._LocaleTime__pad(a_month, True)
        else:
            raise TypeError('abbreviated month names must be a 12-item sequence')
        if am_pm is None:
            self._LocaleTime__am_pm = None
        elif len(am_pm) == 2:
            self._LocaleTime__am_pm = am_pm
        else:
            raise TypeError('AM/PM representation must be a 2-item sequence')
        self._LocaleTime__LC_date_time = LC_date_time
        self._LocaleTime__LC_time = LC_time
        self._LocaleTime__LC_date = LC_date
        self._LocaleTime__timezone = timezone
        if timezone:
            if len(timezone) != 2:
                raise TypeError('timezone names must contain 2 items')
            else:
                self._LocaleTime__timezone = self._LocaleTime__pad(timezone, False)
        
        if lang:
            self._LocaleTime__lang = lang
        else:
            self._LocaleTime__lang = _getlang()

    
    def __pad(self, seq, front):
        seq = list(seq)
        if front:
            seq.insert(0, '')
        else:
            seq.append('')
        return seq

    
    def __set_nothing(self, stuff):
        raise TypeError('attribute does not support assignment')

    
    def __get_f_weekday(self):
        if not (self._LocaleTime__f_weekday):
            self._LocaleTime__calc_weekday()
        
        return self._LocaleTime__f_weekday

    
    def __get_a_weekday(self):
        if not (self._LocaleTime__a_weekday):
            self._LocaleTime__calc_weekday()
        
        return self._LocaleTime__a_weekday

    f_weekday = property(__get_f_weekday, __set_nothing, doc = 'Full weekday names')
    a_weekday = property(__get_a_weekday, __set_nothing, doc = 'Abbreviated weekday names')
    
    def __get_f_month(self):
        if not (self._LocaleTime__f_month):
            self._LocaleTime__calc_month()
        
        return self._LocaleTime__f_month

    
    def __get_a_month(self):
        if not (self._LocaleTime__a_month):
            self._LocaleTime__calc_month()
        
        return self._LocaleTime__a_month

    f_month = property(__get_f_month, __set_nothing, doc = 'Full month names (dummy value at index 0)')
    a_month = property(__get_a_month, __set_nothing, doc = 'Abbreviated month names (dummy value at index 0)')
    
    def __get_am_pm(self):
        if not (self._LocaleTime__am_pm):
            self._LocaleTime__calc_am_pm()
        
        return self._LocaleTime__am_pm

    am_pm = property(__get_am_pm, __set_nothing, doc = 'AM/PM representation')
    
    def __get_timezone(self):
        if not (self._LocaleTime__timezone):
            self._LocaleTime__calc_timezone()
        
        return self._LocaleTime__timezone

    timezone = property(__get_timezone, __set_nothing, doc = 'Timezone representation (dummy value at index 2)')
    
    def __get_LC_date_time(self):
        if not (self._LocaleTime__LC_date_time):
            self._LocaleTime__calc_date_time()
        
        return self._LocaleTime__LC_date_time

    
    def __get_LC_date(self):
        if not (self._LocaleTime__LC_date):
            self._LocaleTime__calc_date_time()
        
        return self._LocaleTime__LC_date

    
    def __get_LC_time(self):
        if not (self._LocaleTime__LC_time):
            self._LocaleTime__calc_date_time()
        
        return self._LocaleTime__LC_time

    LC_date_time = property(__get_LC_date_time, __set_nothing, doc = "Format string for locale's date/time representation ('%c' format)")
    LC_date = property(__get_LC_date, __set_nothing, doc = "Format string for locale's date representation ('%x' format)")
    LC_time = property(__get_LC_time, __set_nothing, doc = "Format string for locale's time representation ('%X' format)")
    lang = property((lambda self: self._LocaleTime__lang), __set_nothing, doc = 'Language used for instance')
    
    def __calc_weekday(self):
        a_weekday = [ calendar.day_abbr[i] for i in range(7) ]
        f_weekday = [ calendar.day_name[i] for i in range(7) ]

    
    def __calc_month(self):
        a_month = [ calendar.month_abbr[i] for i in range(13) ]
        f_month = [ calendar.month_name[i] for i in range(13) ]

    
    def __calc_am_pm(self):
        am_pm = []
        for hour in (1, 22):
            time_tuple = time.struct_time((1999, 3, 17, hour, 44, 55, 2, 76, 0))
            am_pm.append(time.strftime('%p', time_tuple))
        
        self._LocaleTime__am_pm = am_pm

    
    def __calc_date_time(self):
        time_tuple = time.struct_time((1999, 3, 17, 22, 44, 55, 2, 76, 0))
        date_time = [
            None,
            None,
            None]
        date_time[0] = time.strftime('%c', time_tuple)
        date_time[1] = time.strftime('%x', time_tuple)
        date_time[2] = time.strftime('%X', time_tuple)
        for offset, directive in ((0, '%c'), (1, '%x'), (2, '%X')):
            current_format = date_time[offset]
            for old, new in (('%', '%%'), (self.f_weekday[2], '%A'), (self.f_month[3], '%B'), (self.a_weekday[2], '%a'), (self.a_month[3], '%b'), (self.am_pm[1], '%p'), (self.timezone[0], '%Z'), (self.timezone[1], '%Z'), ('1999', '%Y'), ('99', '%y'), ('22', '%H'), ('44', '%M'), ('55', '%S'), ('76', '%j'), ('17', '%d'), ('03', '%m'), ('3', '%m'), ('2', '%w'), ('10', '%I')):
                if old:
                    current_format = current_format.replace(old, new)
                    continue
            
            time_tuple = time.struct_time((1999, 1, 3, 1, 1, 1, 6, 3, 0))
            if time.strftime(directive, time_tuple).find('00'):
                U_W = '%U'
            else:
                U_W = '%W'
            date_time[offset] = current_format.replace('11', U_W)
        
        if not (self._LocaleTime__LC_date_time):
            self._LocaleTime__LC_date_time = date_time[0]
        
        if not (self._LocaleTime__LC_date):
            self._LocaleTime__LC_date = date_time[1]
        
        if not (self._LocaleTime__LC_time):
            self._LocaleTime__LC_time = date_time[2]
        

    
    def __calc_timezone(self):
        
        try:
            time.tzset()
        except AttributeError:
            pass

        time_zones = [
            'UTC',
            'GMT']
        if time.daylight:
            time_zones.extend(time.tzname)
        else:
            time_zones.append(time.tzname[0])
        self._LocaleTime__timezone = self._LocaleTime__pad(time_zones, 0)



class TimeRE(dict):
    '''Handle conversion from format directives to regexes.'''
    
    def __init__(self, locale_time = None):
        '''Init inst with non-locale regexes and store LocaleTime object.'''
        base = super(TimeRE, self)
        base.__init__({
            'd': '(?P<d>3[0-1]|[1-2]\\d|0[1-9]|[1-9]| [1-9])',
            'H': '(?P<H>2[0-3]|[0-1]\\d|\\d)',
            'I': '(?P<I>1[0-2]|0[1-9]|[1-9])',
            'j': '(?P<j>36[0-6]|3[0-5]\\d|[1-2]\\d\\d|0[1-9]\\d|00[1-9]|[1-9]\\d|0[1-9]|[1-9])',
            'm': '(?P<m>1[0-2]|0[1-9]|[1-9])',
            'M': '(?P<M>[0-5]\\d|\\d)',
            'S': '(?P<S>6[0-1]|[0-5]\\d|\\d)',
            'U': '(?P<U>5[0-3]|[0-4]\\d|\\d)',
            'w': '(?P<w>[0-6])',
            'y': '(?P<y>\\d\\d)',
            'Y': '(?P<Y>\\d\\d\\d\\d)' })
        base.__setitem__('W', base.__getitem__('U'))
        if locale_time:
            self.locale_time = locale_time
        else:
            self.locale_time = LocaleTime()

    
    def __getitem__(self, fetch):
        '''Try to fetch regex; if it does not exist, construct it.'''
        
        try:
            return super(TimeRE, self).__getitem__(fetch)
        except KeyError:
            constructors = {
                'A': (lambda : self._TimeRE__seqToRE(self.locale_time.f_weekday, fetch)),
                'a': (lambda : self._TimeRE__seqToRE(self.locale_time.a_weekday, fetch)),
                'B': (lambda : self._TimeRE__seqToRE(self.locale_time.f_month[1:], fetch)),
                'b': (lambda : self._TimeRE__seqToRE(self.locale_time.a_month[1:], fetch)),
                'c': (lambda : self.pattern(self.locale_time.LC_date_time)),
                'p': (lambda : self._TimeRE__seqToRE(self.locale_time.am_pm, fetch)),
                'x': (lambda : self.pattern(self.locale_time.LC_date)),
                'X': (lambda : self.pattern(self.locale_time.LC_time)),
                'Z': (lambda : self._TimeRE__seqToRE(self.locale_time.timezone, fetch)),
                '%': (lambda : '%') }
            if fetch in constructors:
                self[fetch] = constructors[fetch]()
                return self[fetch]
            else:
                raise 
        except:
            fetch in constructors


    
    def __seqToRE(self, to_convert, directive):
        '''Convert a list to a regex string for matching a directive.'''
        
        def sorter(a, b):
            '''Sort based on length.

            Done in case for some strange reason that names in the locale only
            differ by a suffix and thus want the name with the suffix to match
            first.
            '''
            
            try:
                a_length = len(a)
            except TypeError:
                a_length = 0

            
            try:
                b_length = len(b)
            except TypeError:
                b_length = 0

            return cmp(b_length, a_length)

        to_convert = to_convert[:]
        for value in to_convert:
            if value != '':
                break
                continue
        else:
            return ''
        to_convert.sort(sorter)
        regex = '|'.join(to_convert)
        regex = '(?P<%s>%s' % (directive, regex)
        return '%s)' % regex

    
    def pattern(self, format):
        '''Return re pattern for the format string.

        Need to make sure that any characters that might be interpreted as
        regex syntax is escaped.

        '''
        processed_format = ''
        regex_chars = re_compile('([\\\\.^$*+?{}\\[\\]|])')
        format = regex_chars.sub('\\\\\\1', format)
        whitespace_replacement = re_compile('\\s+')
        format = whitespace_replacement.sub('\\s*', format)
        while format.find('%') != -1:
            directive_index = format.index('%') + 1
            processed_format = '%s%s%s' % (processed_format, format[:directive_index - 1], self[format[directive_index]])
            format = format[directive_index + 1:]
        return '%s%s' % (processed_format, format)

    
    def compile(self, format):
        '''Return a compiled re object for the format string.'''
        return re_compile(self.pattern(format), IGNORECASE)



def strptime(data_string, format = '%a %b %d %H:%M:%S %Y'):
    '''Return a time struct based on the input data and the format string.'''
    time_re = TimeRE()
    locale_time = time_re.locale_time
    format_regex = time_re.compile(format)
    found = format_regex.match(data_string)
    if not found:
        raise ValueError('time data did not match format:  data=%s  fmt=%s' % (data_string, format))
    
    if len(data_string) != found.end():
        raise ValueError('unconverted data remains: %s' % data_string[found.end():])
    
    year = 1900
    month = day = 1
    hour = minute = second = 0
    tz = -1
    weekday = julian = -1
    found_dict = found.groupdict()
    for group_key in found_dict.iterkeys():
        if group_key == 'y':
            year = int(found_dict['y'])
            if year <= 68:
                year += 2000
            else:
                year += 1900
        year <= 68
        if group_key == 'Y':
            year = int(found_dict['Y'])
            continue
        if group_key == 'm':
            month = int(found_dict['m'])
            continue
        if group_key == 'B':
            month = _insensitiveindex(locale_time.f_month, found_dict['B'])
            continue
        if group_key == 'b':
            month = _insensitiveindex(locale_time.a_month, found_dict['b'])
            continue
        if group_key == 'd':
            day = int(found_dict['d'])
            continue
        if group_key == 'H':
            hour = int(found_dict['H'])
            continue
        if group_key == 'I':
            hour = int(found_dict['I'])
            ampm = found_dict.get('p', '').lower()
            if ampm in ('', locale_time.am_pm[0].lower()):
                if hour == 12:
                    hour = 0
                
            elif ampm == locale_time.am_pm[1].lower():
                if hour != 12:
                    hour += 12
                
            
        ampm in ('', locale_time.am_pm[0].lower())
        if group_key == 'M':
            minute = int(found_dict['M'])
            continue
        if group_key == 'S':
            second = int(found_dict['S'])
            continue
        if group_key == 'A':
            weekday = _insensitiveindex(locale_time.f_weekday, found_dict['A'])
            continue
        if group_key == 'a':
            weekday = _insensitiveindex(locale_time.a_weekday, found_dict['a'])
            continue
        if group_key == 'w':
            weekday = int(found_dict['w'])
            if weekday == 0:
                weekday = 6
            else:
                weekday -= 1
        weekday == 0
        if group_key == 'j':
            julian = int(found_dict['j'])
            continue
        if group_key == 'Z':
            found_zone = found_dict['Z'].lower()
            if locale_time.timezone[0] == locale_time.timezone[1] and time.daylight:
                pass
            elif found_zone in ('utc', 'gmt'):
                tz = 0
            elif locale_time.timezone[2].lower() == found_zone:
                tz = 0
            elif time.daylight and locale_time.timezone[3].lower() == found_zone:
                tz = 1
            
        locale_time.timezone[3].lower() == found_zone
    
    if julian == -1:
        julian = (datetime_date(year, month, day).toordinal() - datetime_date(year, 1, 1).toordinal()) + 1
    else:
        datetime_result = datetime_date.fromordinal((julian - 1) + datetime_date(year, 1, 1).toordinal())
        year = datetime_result.year
        month = datetime_result.month
        day = datetime_result.day
    if weekday == -1:
        weekday = datetime_date(year, month, day).weekday()
    
    return time.struct_time((year, month, day, hour, minute, second, weekday, julian, tz))


def _insensitiveindex(lst, findme):
    findme = findme.lower()
    for key, item in enumerate(lst):
        if item.lower() == findme:
            return key
            continue
    else:
        raise ValueError('value not in list')

