# Source Generated with Decompyle++
# File: in.pyo (Python 2.3)

'''
Basic TestCases for BTree and hash DBs, with and without a DBEnv, with
various DB flags, etc.
'''
import os
import sys
import errno
import shutil
import string
import tempfile
from pprint import pprint
import unittest

try:
    from bsddb import db
except ImportError:
    from bsddb3 import db

from test_all import verbose
DASH = '-'

class VersionTestCase(unittest.TestCase):
    
    def test00_version(self):
        info = db.version()
        if verbose:
            print '\n', '-=' * 20
            print 'bsddb.db.version(): %s' % (info,)
            print db.DB_VERSION_STRING
            print '-=' * 20
        



class BasicTestCase(unittest.TestCase):
    dbtype = db.DB_UNKNOWN
    dbopenflags = 0
    dbsetflags = 0
    dbmode = 432
    dbname = None
    useEnv = 0
    envflags = 0
    envsetflags = 0
    
    def setUp(self):
        if self.useEnv:
            homeDir = os.path.join(os.path.dirname(sys.argv[0]), 'db_home')
            self.homeDir = homeDir
            
            try:
                shutil.rmtree(homeDir)
            except OSError:
                e = None
                if e.errno not in (errno.ENOENT, errno.ESRCH):
                    raise 
                
            except:
                e.errno not in (errno.ENOENT, errno.ESRCH)

            os.mkdir(homeDir)
            
            try:
                self.env = db.DBEnv()
                self.env.set_lg_max(1024 * 1024)
                self.env.set_flags(self.envsetflags, 1)
                self.env.open(homeDir, self.envflags | db.DB_CREATE)
                tempfile.tempdir = homeDir
                self.filename = os.path.split(tempfile.mktemp())[1]
                tempfile.tempdir = None
            shutil.rmtree(homeDir)
            raise 

        else:
            self.env = None
            self.filename = tempfile.mktemp()
        self.d = db.DB(self.env)
        self.d.set_flags(self.dbsetflags)
        if self.dbname:
            self.d.open(self.filename, self.dbname, self.dbtype, self.dbopenflags | db.DB_CREATE, self.dbmode)
        else:
            self.d.open(self.filename, mode = self.dbmode, dbtype = self.dbtype, flags = self.dbopenflags | db.DB_CREATE)
        self.populateDB()

    
    def tearDown(self):
        self.d.close()
        if self.env is not None:
            self.env.close()
            shutil.rmtree(self.homeDir)
        else:
            os.remove(self.filename)

    
    def populateDB(self):
        d = self.d
        for x in range(500):
            key = '%04d' % (1000 - x)
            data = self.makeData(key)
            d.put(key, data)
        
        for x in range(500):
            key = '%04d' % x
            data = self.makeData(key)
            d.put(key, data)
        
        num = len(d)
        if verbose:
            print 'created %d records' % num
        

    
    def makeData(self, key):
        return DASH.join([
            key] * 5)

    
    def test01_GetsAndPuts(self):
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test01_GetsAndPuts...' % self.__class__.__name__
        
        for key in [
            '0001',
            '0100',
            '0400',
            '0700',
            '0999']:
            data = d.get(key)
            if verbose:
                print data
                continue
        
        
        try:
            d.delete('abcd')
        except db.DBNotFoundError:
            val = None
            if verbose:
                print val
            
        except:
            verbose

        self.fail('expected exception')
        d.put('abcd', 'a new record')
        d.put('abcd', 'same key')
        if self.dbsetflags & db.DB_DUP:
            pass
        
        
        try:
            d.put('abcd', 'this should fail', flags = db.DB_NOOVERWRITE)
        except db.DBKeyExistError:
            val = None
            if verbose:
                print val
            
        except:
            verbose

        self.fail('expected exception')
        if self.dbsetflags & db.DB_DUP:
            pass
        
        d.sync()
        d.close()
        del d
        self.d = db.DB(self.env)
        if self.dbname:
            self.d.open(self.filename, self.dbname)
        else:
            self.d.open(self.filename)
        d = self.d
        if self.dbsetflags & db.DB_DUP:
            pass
        
        rec = d.get_both('0555', '0555-0555-0555-0555-0555')
        if verbose:
            print rec
        
        data = d.get('bad key', 'bad data')
        data = d.get('bad key', self)
        s = d.stat()
        if verbose:
            print 'd.stat() returned this dictionary:'
            pprint(s)
        

    
    def test02_DictionaryMethods(self):
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test02_DictionaryMethods...' % self.__class__.__name__
        
        for key in [
            '0002',
            '0101',
            '0401',
            '0701',
            '0998']:
            data = d[key]
            if verbose:
                print data
                continue
        
        keys = d.keys()
        d['new record'] = 'a new record'
        keys = d.keys()
        d['new record'] = 'a replacement record'
        keys = d.keys()
        if verbose:
            print 'the first 10 keys are:'
            pprint(keys[:10])
        
        items = d.items()
        if verbose:
            print 'the first 10 items are:'
            pprint(items[:10])
        
        values = d.values()
        if verbose:
            print 'the first 10 values are:'
            pprint(values[:10])
        

    
    def test03_SimpleCursorStuff(self, get_raises_error = 0, set_raises_error = 1):
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test03_SimpleCursorStuff (get_error %s, set_error %s)...' % (self.__class__.__name__, get_raises_error, set_raises_error)
        
        if self.env and self.dbopenflags & db.DB_AUTO_COMMIT:
            txn = self.env.txn_begin()
        else:
            txn = None
        c = self.d.cursor(txn = txn)
        rec = c.first()
        count = 0
        while rec is not None:
            count = count + 1
            if verbose and count % 100 == 0:
                print rec
            
            
            try:
                rec = c.next()
            continue
            except db.DBNotFoundError:
                val = None
                if get_raises_error:
                    if verbose:
                        print val
                    
                    rec = None
                else:
                    self.fail('unexpected DBNotFoundError')
                get_raises_error
            

            None<EXCEPTION MATCH>db.DBNotFoundError
        rec = c.last()
        count = 0
        while rec is not None:
            count = count + 1
            if verbose and count % 100 == 0:
                print rec
            
            
            try:
                rec = c.prev()
            continue
            except db.DBNotFoundError:
                val = None
                if get_raises_error:
                    if verbose:
                        print val
                    
                    rec = None
                else:
                    self.fail('unexpected DBNotFoundError')
                get_raises_error
            

            None<EXCEPTION MATCH>db.DBNotFoundError
        rec = c.set('0505')
        rec2 = c.current()
        
        try:
            n = c.set('bad key')
        except db.DBNotFoundError:
            val = None
            if verbose:
                print val
            
        except:
            verbose

        if set_raises_error:
            self.fail('expected exception')
        
        if n != None:
            self.fail('expected None: ' + `n`)
        
        rec = c.get_both('0404', self.makeData('0404'))
        
        try:
            n = c.get_both('0404', 'bad data')
        except db.DBNotFoundError:
            val = None
            if verbose:
                print val
            
        except:
            verbose

        if get_raises_error:
            self.fail('expected exception')
        
        if n != None:
            self.fail('expected None: ' + `n`)
        
        if self.d.get_type() == db.DB_BTREE:
            rec = c.set_range('011')
            if verbose:
                print "searched for '011', found: ", rec
            
            rec = c.set_range('011', dlen = 0, doff = 0)
            if verbose:
                print "searched (partial) for '011', found: ", rec
            
            if rec[1] != '':
                set.fail('expected empty data portion')
            
        
        c.set('0499')
        c.delete()
        
        try:
            rec = c.current()
        except db.DBKeyEmptyError:
            val = None
            if verbose:
                print val
            
        except:
            verbose

        self.fail('exception expected')
        c.next()
        c2 = c.dup(db.DB_POSITION)
        c2.put('', 'a new value', db.DB_CURRENT)
        c2.put('', 'er', db.DB_CURRENT, dlen = 0, doff = 5)
        c.close()
        c2.close()
        if txn:
            txn.commit()
        
        methods_to_test = {
            'current': (),
            'delete': (),
            'dup': (db.DB_POSITION,),
            'first': (),
            'get': (0,),
            'next': (),
            'prev': (),
            'last': (),
            'put': ('', 'spam', db.DB_CURRENT),
            'set': ('0505',) }
        for method, args in methods_to_test.items():
            
            try:
                if verbose:
                    print "attempting to use a closed cursor's %s method" % method
                
                apply(getattr(c, method), args)
            except db.DBError:
                val = None
                if verbose:
                    print val
                
                verbose

            self.fail("no exception raised when using a buggy cursor's%s method" % method)
        
        oldcursor = self.d.cursor(txn = txn)
        self.d.close()
        del oldcursor

    
    def test03b_SimpleCursorWithoutGetReturnsNone0(self):
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test03b_SimpleCursorStuffWithoutGetReturnsNone...' % self.__class__.__name__
        
        old = self.d.set_get_returns_none(0)
        self.test03_SimpleCursorStuff(get_raises_error = 1, set_raises_error = 1)

    
    def test03c_SimpleCursorGetReturnsNone2(self):
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test03c_SimpleCursorStuffWithoutSetReturnsNone...' % self.__class__.__name__
        
        old = self.d.set_get_returns_none(2)
        old = self.d.set_get_returns_none(2)
        self.test03_SimpleCursorStuff(get_raises_error = 0, set_raises_error = 0)

    
    def test04_PartialGetAndPut(self):
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test04_PartialGetAndPut...' % self.__class__.__name__
        
        key = 'partialTest'
        data = '1' * 1000 + '2' * 1000
        d.put(key, data)
        d.put('partialtest2', '1' * 30000 + 'robin')
        if self.dbsetflags != db.DB_DUP:
            d.put(key, '0000', dlen = 2000, doff = 0)
            d.put(key, '1111', dlen = 1, doff = 2)
        

    
    def test05_GetSize(self):
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test05_GetSize...' % self.__class__.__name__
        
        for i in range(1, 50000, 500):
            key = 'size%s' % i
            d.put(key, '1' * i)
        

    
    def test06_Truncate(self):
        if db.version() < (3, 3):
            return None
        
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test99_Truncate...' % self.__class__.__name__
        
        d.put('abcde', 'ABCDE')
        num = d.truncate()
        num = d.truncate()



class BasicBTreeTestCase(BasicTestCase):
    dbtype = db.DB_BTREE


class BasicHashTestCase(BasicTestCase):
    dbtype = db.DB_HASH


class BasicBTreeWithThreadFlagTestCase(BasicTestCase):
    dbtype = db.DB_BTREE
    dbopenflags = db.DB_THREAD


class BasicHashWithThreadFlagTestCase(BasicTestCase):
    dbtype = db.DB_HASH
    dbopenflags = db.DB_THREAD


class BasicBTreeWithEnvTestCase(BasicTestCase):
    dbtype = db.DB_BTREE
    dbopenflags = db.DB_THREAD
    useEnv = 1
    envflags = db.DB_THREAD | db.DB_INIT_MPOOL | db.DB_INIT_LOCK


class BasicHashWithEnvTestCase(BasicTestCase):
    dbtype = db.DB_HASH
    dbopenflags = db.DB_THREAD
    useEnv = 1
    envflags = db.DB_THREAD | db.DB_INIT_MPOOL | db.DB_INIT_LOCK


class BasicTransactionTestCase(BasicTestCase):
    dbopenflags = db.DB_THREAD | db.DB_AUTO_COMMIT
    useEnv = 1
    envflags = db.DB_THREAD | db.DB_INIT_MPOOL | db.DB_INIT_LOCK | db.DB_INIT_TXN
    envsetflags = db.DB_AUTO_COMMIT
    
    def tearDown(self):
        self.txn.commit()
        BasicTestCase.tearDown(self)

    
    def populateDB(self):
        d = self.d
        txn = self.env.txn_begin()
        for x in range(500):
            key = '%04d' % (1000 - x)
            data = self.makeData(key)
            d.put(key, data, txn)
        
        for x in range(500):
            key = '%04d' % x
            data = self.makeData(key)
            d.put(key, data, txn)
        
        txn.commit()
        num = len(d)
        if verbose:
            print 'created %d records' % num
        
        self.txn = self.env.txn_begin()

    
    def test06_Transactions(self):
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test06_Transactions...' % self.__class__.__name__
        
        d.put('new rec', 'this is a new record', self.txn)
        self.txn.abort()
        self.txn = self.env.txn_begin()
        d.put('new rec', 'this is a new record', self.txn)
        self.txn.commit()
        self.txn = self.env.txn_begin()
        c = d.cursor(self.txn)
        rec = c.first()
        count = 0
        while rec is not None:
            count = count + 1
            if verbose and count % 100 == 0:
                print rec
            
            rec = c.next()
        c.close()
        self.txn.commit()
        
        try:
            self.env.txn_checkpoint(0, 0, 0)
        except db.DBIncompleteError:
            pass

        logs = self.env.log_archive(db.DB_ARCH_ABS | db.DB_ARCH_LOG)
        for log in logs:
            if verbose:
                print 'log file: ' + log
                continue
        
        self.txn = self.env.txn_begin()

    
    def test07_TxnTruncate(self):
        if db.version() < (3, 3):
            return None
        
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test07_TxnTruncate...' % self.__class__.__name__
        
        d.put('abcde', 'ABCDE')
        txn = self.env.txn_begin()
        num = d.truncate(txn)
        num = d.truncate(txn)
        txn.commit()

    
    def test08_TxnLateUse(self):
        txn = self.env.txn_begin()
        txn.abort()
        
        try:
            txn.abort()
        except db.DBError:
            e = None

        raise RuntimeError, 'DBTxn.abort() called after DB_TXN no longer valid w/o an exception'
        txn = self.env.txn_begin()
        txn.commit()
        
        try:
            txn.commit()
        except db.DBError:
            e = None

        raise RuntimeError, 'DBTxn.commit() called after DB_TXN no longer valid w/o an exception'



class BTreeTransactionTestCase(BasicTransactionTestCase):
    dbtype = db.DB_BTREE


class HashTransactionTestCase(BasicTransactionTestCase):
    dbtype = db.DB_HASH


class BTreeRecnoTestCase(BasicTestCase):
    dbtype = db.DB_BTREE
    dbsetflags = db.DB_RECNUM
    
    def test07_RecnoInBTree(self):
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test07_RecnoInBTree...' % self.__class__.__name__
        
        rec = d.get(200)
        if verbose:
            print 'Record #200 is ', rec
        
        c = d.cursor()
        c.set('0200')
        num = c.get_recno()
        if verbose:
            print "recno of d['0200'] is ", num
        
        rec = c.current()
        c.close()



class BTreeRecnoWithThreadFlagTestCase(BTreeRecnoTestCase):
    dbopenflags = db.DB_THREAD


class BasicDUPTestCase(BasicTestCase):
    dbsetflags = db.DB_DUP
    
    def test08_DuplicateKeys(self):
        d = self.d
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test08_DuplicateKeys...' % self.__class__.__name__
        
        d.put('dup0', 'before')
        for x in 'The quick brown fox jumped over the lazy dog.'.split():
            d.put('dup1', x)
        
        d.put('dup2', 'after')
        data = d.get('dup1')
        if verbose:
            print data
        
        c = d.cursor()
        rec = c.set('dup1')
        next = c.next()
        rec = c.set('dup1')
        count = c.count()
        next_dup = c.next_dup()
        rec = c.set('dup1')
        while rec is not None:
            if verbose:
                print rec
            
            rec = c.next_dup()
        c.set('dup1')
        rec = c.next_nodup()
        if verbose:
            print rec
        
        c.close()



class BTreeDUPTestCase(BasicDUPTestCase):
    dbtype = db.DB_BTREE


class HashDUPTestCase(BasicDUPTestCase):
    dbtype = db.DB_HASH


class BTreeDUPWithThreadTestCase(BasicDUPTestCase):
    dbtype = db.DB_BTREE
    dbopenflags = db.DB_THREAD


class HashDUPWithThreadTestCase(BasicDUPTestCase):
    dbtype = db.DB_HASH
    dbopenflags = db.DB_THREAD


class BasicMultiDBTestCase(BasicTestCase):
    dbname = 'first'
    
    def otherType(self):
        if self.dbtype == db.DB_BTREE:
            return db.DB_HASH
        else:
            return db.DB_BTREE

    
    def test09_MultiDB(self):
        d1 = self.d
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test09_MultiDB...' % self.__class__.__name__
        
        d2 = db.DB(self.env)
        d2.open(self.filename, 'second', self.dbtype, self.dbopenflags | db.DB_CREATE)
        d3 = db.DB(self.env)
        d3.open(self.filename, 'third', self.otherType(), self.dbopenflags | db.DB_CREATE)
        for x in 'The quick brown fox jumped over the lazy dog'.split():
            d2.put(x, self.makeData(x))
        
        for x in string.letters:
            d3.put(x, x * 70)
        
        d1.sync()
        d2.sync()
        d3.sync()
        d1.close()
        d2.close()
        d3.close()
        self.d = d1 = d2 = d3 = None
        self.d = d1 = db.DB(self.env)
        d1.open(self.filename, self.dbname, flags = self.dbopenflags)
        d2 = db.DB(self.env)
        d2.open(self.filename, 'second', flags = self.dbopenflags)
        d3 = db.DB(self.env)
        d3.open(self.filename, 'third', flags = self.dbopenflags)
        c1 = d1.cursor()
        c2 = d2.cursor()
        c3 = d3.cursor()
        count = 0
        rec = c1.first()
        while rec is not None:
            count = count + 1
            if verbose and count % 50 == 0:
                print rec
            
            rec = c1.next()
        count = 0
        rec = c2.first()
        while rec is not None:
            count = count + 1
            if verbose:
                print rec
            
            rec = c2.next()
        count = 0
        rec = c3.first()
        while rec is not None:
            count = count + 1
            if verbose:
                print rec
            
            rec = c3.next()
        c1.close()
        c2.close()
        c3.close()
        d2.close()
        d3.close()



class BTreeMultiDBTestCase(BasicMultiDBTestCase):
    dbtype = db.DB_BTREE
    dbopenflags = db.DB_THREAD
    useEnv = 1
    envflags = db.DB_THREAD | db.DB_INIT_MPOOL | db.DB_INIT_LOCK


class HashMultiDBTestCase(BasicMultiDBTestCase):
    dbtype = db.DB_HASH
    dbopenflags = db.DB_THREAD
    useEnv = 1
    envflags = db.DB_THREAD | db.DB_INIT_MPOOL | db.DB_INIT_LOCK


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(VersionTestCase))
    suite.addTest(unittest.makeSuite(BasicBTreeTestCase))
    suite.addTest(unittest.makeSuite(BasicHashTestCase))
    suite.addTest(unittest.makeSuite(BasicBTreeWithThreadFlagTestCase))
    suite.addTest(unittest.makeSuite(BasicHashWithThreadFlagTestCase))
    suite.addTest(unittest.makeSuite(BasicBTreeWithEnvTestCase))
    suite.addTest(unittest.makeSuite(BasicHashWithEnvTestCase))
    suite.addTest(unittest.makeSuite(BTreeTransactionTestCase))
    suite.addTest(unittest.makeSuite(HashTransactionTestCase))
    suite.addTest(unittest.makeSuite(BTreeRecnoTestCase))
    suite.addTest(unittest.makeSuite(BTreeRecnoWithThreadFlagTestCase))
    suite.addTest(unittest.makeSuite(BTreeDUPTestCase))
    suite.addTest(unittest.makeSuite(HashDUPTestCase))
    suite.addTest(unittest.makeSuite(BTreeDUPWithThreadTestCase))
    suite.addTest(unittest.makeSuite(HashDUPWithThreadTestCase))
    suite.addTest(unittest.makeSuite(BTreeMultiDBTestCase))
    suite.addTest(unittest.makeSuite(HashMultiDBTestCase))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest = 'test_suite')

