# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

import sys
import os
import string
import unittest
import glob

try:
    from bsddb import db, dbobj
except ImportError:
    from bsddb3 import db, dbobj


class dbobjTestCase(unittest.TestCase):
    '''Verify that dbobj.DB and dbobj.DBEnv work properly'''
    db_home = 'db_home'
    db_name = 'test-dbobj.db'
    
    def setUp(self):
        homeDir = os.path.join(os.path.dirname(sys.argv[0]), 'db_home')
        self.homeDir = homeDir
        
        try:
            os.mkdir(homeDir)
        except os.error:
            pass


    
    def tearDown(self):
        if hasattr(self, 'db'):
            del self.db
        
        if hasattr(self, 'env'):
            del self.env
        
        files = glob.glob(os.path.join(self.homeDir, '*'))
        for file in files:
            os.remove(file)
        

    
    def test01_both(self):
        
        class TestDBEnv(dbobj.DBEnv):
            pass

        
        class TestDB(dbobj.DB):
            
            def put(self, key, *args, **kwargs):
                key = string.upper(key)
                return apply(dbobj.DB.put, (self, key) + args, kwargs)


        self.env = TestDBEnv()
        self.env.open(self.homeDir, db.DB_CREATE | db.DB_INIT_MPOOL)
        self.db = TestDB(self.env)
        self.db.open(self.db_name, db.DB_HASH, db.DB_CREATE)
        self.db.put('spam', 'eggs')
        if not self.db.get('spam') == None:
            raise AssertionError, 'overridden dbobj.DB.put() method failed [1]'
        if not self.db.get('SPAM') == 'eggs':
            raise AssertionError, 'overridden dbobj.DB.put() method failed [2]'
        self.db.close()
        self.env.close()

    
    def test02_dbobj_dict_interface(self):
        self.env = dbobj.DBEnv()
        self.env.open(self.homeDir, db.DB_CREATE | db.DB_INIT_MPOOL)
        self.db = dbobj.DB(self.env)
        self.db.open(self.db_name + '02', db.DB_HASH, db.DB_CREATE)
        self.db['spam'] = 'eggs'
        if not len(self.db) == 1:
            raise AssertionError
        if not self.db['spam'] == 'eggs':
            raise AssertionError
        del self.db['spam']
        if not self.db.get('spam') == None:
            raise AssertionError, 'dbobj __del__ failed'
        self.db.close()
        self.env.close()



def test_suite():
    return unittest.makeSuite(dbobjTestCase)

if __name__ == '__main__':
    unittest.main(defaultTest = 'test_suite')

