# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

'''
TestCases for checking dbShelve objects.
'''
import sys
import os
import string
import tempfile
import random
from pprint import pprint
from types import *
import unittest

try:
    from bsddb import db, dbshelve
except ImportError:
    from bsddb3 import db, dbshelve

from test_all import verbose

class DataClass:
    
    def __init__(self):
        self.value = random.random()

    
    def __cmp__(self, other):
        return cmp(self.value, other)



class DBShelveTestCase(unittest.TestCase):
    
    def setUp(self):
        self.filename = tempfile.mktemp()
        self.do_open()

    
    def tearDown(self):
        self.do_close()
        
        try:
            os.remove(self.filename)
        except os.error:
            pass


    
    def populateDB(self, d):
        for x in string.letters:
            d['S' + x] = 10 * x
            d['I' + x] = ord(x)
            d['L' + x] = [
                x] * 10
            inst = DataClass()
            inst.S = 10 * x
            inst.I = ord(x)
            inst.L = [
                x] * 10
            d['O' + x] = inst
        

    
    def do_open(self):
        self.d = dbshelve.open(self.filename)

    
    def do_close(self):
        self.d.close()

    
    def test01_basics(self):
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test01_basics...' % self.__class__.__name__
        
        self.populateDB(self.d)
        self.d.sync()
        self.do_close()
        self.do_open()
        d = self.d
        l = len(d)
        k = d.keys()
        s = d.stat()
        f = d.fd()
        if verbose:
            print 'length:', l
            print 'keys:', k
            print 'stats:', s
        
        if not 0 == d.has_key('bad key'):
            raise AssertionError
        if not 1 == d.has_key('IA'):
            raise AssertionError
        if not 1 == d.has_key('OA'):
            raise AssertionError
        d.delete('IA')
        del d['OA']
        if not 0 == d.has_key('IA'):
            raise AssertionError
        if not 0 == d.has_key('OA'):
            raise AssertionError
        if not len(d) == l - 2:
            raise AssertionError
        values = []
        for key in d.keys():
            value = d[key]
            values.append(value)
            if verbose:
                print '%s: %s' % (key, value)
            
            self.checkrec(key, value)
        
        dbvalues = d.values()
        if not len(dbvalues) == len(d.keys()):
            raise AssertionError
        values.sort()
        dbvalues.sort()
        if not values == dbvalues:
            raise AssertionError
        items = d.items()
        if not len(items) == len(values):
            raise AssertionError
        for key, value in items:
            self.checkrec(key, value)
        
        if not d.get('bad key') == None:
            raise AssertionError
        if not d.get('bad key', None) == None:
            raise AssertionError
        if not d.get('bad key', 'a string') == 'a string':
            raise AssertionError
        if not d.get('bad key', [
            1,
            2,
            3]) == [
            1,
            2,
            3]:
            raise AssertionError
        d.set_get_returns_none(0)
        self.assertRaises(db.DBNotFoundError, d.get, 'bad key')
        d.set_get_returns_none(1)
        d.put('new key', 'new data')
        if not d.get('new key') == 'new data':
            raise AssertionError
        if not d['new key'] == 'new data':
            raise AssertionError

    
    def test02_cursors(self):
        if verbose:
            print '\n', '-=' * 30
            print 'Running %s.test02_cursors...' % self.__class__.__name__
        
        self.populateDB(self.d)
        d = self.d
        count = 0
        c = d.cursor()
        rec = c.first()
        while rec is not None:
            count = count + 1
            if verbose:
                print rec
            
            (key, value) = rec
            self.checkrec(key, value)
            rec = c.next()
        del c
        if not count == len(d):
            raise AssertionError
        count = 0
        c = d.cursor()
        rec = c.last()
        while rec is not None:
            count = count + 1
            if verbose:
                print rec
            
            (key, value) = rec
            self.checkrec(key, value)
            rec = c.prev()
        if not count == len(d):
            raise AssertionError
        c.set('SS')
        (key, value) = c.current()
        self.checkrec(key, value)
        del c

    
    def checkrec(self, key, value):
        x = key[1]
        if key[0] == 'S':
            if not type(value) == StringType:
                raise AssertionError
            if not value == 10 * x:
                raise AssertionError
        elif key[0] == 'I':
            if not type(value) == IntType:
                raise AssertionError
            if not value == ord(x):
                raise AssertionError
        elif key[0] == 'L':
            if not type(value) == ListType:
                raise AssertionError
            if not value == [
                x] * 10:
                raise AssertionError
        elif key[0] == 'O':
            if not type(value) == InstanceType:
                raise AssertionError
            if not value.S == 10 * x:
                raise AssertionError
            if not value.I == ord(x):
                raise AssertionError
            if not value.L == [
                x] * 10:
                raise AssertionError
        else:
            raise AssertionError, 'Unknown key type, fix the test'



class BasicShelveTestCase(DBShelveTestCase):
    
    def do_open(self):
        self.d = dbshelve.DBShelf()
        self.d.open(self.filename, self.dbtype, self.dbflags)

    
    def do_close(self):
        self.d.close()



class BTreeShelveTestCase(BasicShelveTestCase):
    dbtype = db.DB_BTREE
    dbflags = db.DB_CREATE


class HashShelveTestCase(BasicShelveTestCase):
    dbtype = db.DB_HASH
    dbflags = db.DB_CREATE


class ThreadBTreeShelveTestCase(BasicShelveTestCase):
    dbtype = db.DB_BTREE
    dbflags = db.DB_CREATE | db.DB_THREAD


class ThreadHashShelveTestCase(BasicShelveTestCase):
    dbtype = db.DB_HASH
    dbflags = db.DB_CREATE | db.DB_THREAD


class BasicEnvShelveTestCase(DBShelveTestCase):
    
    def do_open(self):
        self.homeDir = homeDir = os.path.join(os.path.dirname(sys.argv[0]), 'db_home')
        
        try:
            os.mkdir(homeDir)
        except os.error:
            pass

        self.env = db.DBEnv()
        self.env.open(homeDir, self.envflags | db.DB_INIT_MPOOL | db.DB_CREATE)
        self.filename = os.path.split(self.filename)[1]
        self.d = dbshelve.DBShelf(self.env)
        self.d.open(self.filename, self.dbtype, self.dbflags)

    
    def do_close(self):
        self.d.close()
        self.env.close()

    
    def tearDown(self):
        self.do_close()
        import glob
        files = glob.glob(os.path.join(self.homeDir, '*'))
        for file in files:
            os.remove(file)
        



class EnvBTreeShelveTestCase(BasicEnvShelveTestCase):
    envflags = 0
    dbtype = db.DB_BTREE
    dbflags = db.DB_CREATE


class EnvHashShelveTestCase(BasicEnvShelveTestCase):
    envflags = 0
    dbtype = db.DB_HASH
    dbflags = db.DB_CREATE


class EnvThreadBTreeShelveTestCase(BasicEnvShelveTestCase):
    envflags = db.DB_THREAD
    dbtype = db.DB_BTREE
    dbflags = db.DB_CREATE | db.DB_THREAD


class EnvThreadHashShelveTestCase(BasicEnvShelveTestCase):
    envflags = db.DB_THREAD
    dbtype = db.DB_HASH
    dbflags = db.DB_CREATE | db.DB_THREAD


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(DBShelveTestCase))
    suite.addTest(unittest.makeSuite(BTreeShelveTestCase))
    suite.addTest(unittest.makeSuite(HashShelveTestCase))
    suite.addTest(unittest.makeSuite(ThreadBTreeShelveTestCase))
    suite.addTest(unittest.makeSuite(ThreadHashShelveTestCase))
    suite.addTest(unittest.makeSuite(EnvBTreeShelveTestCase))
    suite.addTest(unittest.makeSuite(EnvHashShelveTestCase))
    suite.addTest(unittest.makeSuite(EnvThreadBTreeShelveTestCase))
    suite.addTest(unittest.makeSuite(EnvThreadHashShelveTestCase))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest = 'test_suite')

