# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

'''TestCases for exercising a Recno DB.
'''
import os
import sys
import errno
import tempfile
from pprint import pprint
import unittest
from test_all import verbose

try:
    from bsddb import db
except ImportError:
    from bsddb3 import db

letters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'

class SimpleRecnoTestCase(unittest.TestCase):
    
    def setUp(self):
        self.filename = tempfile.mktemp()

    
    def tearDown(self):
        
        try:
            os.remove(self.filename)
        except OSError:
            e = None
            if e.errno != errno.EEXIST:
                raise 
            
        except:
            e.errno != errno.EEXIST


    
    def test01_basic(self):
        d = db.DB()
        d.open(self.filename, db.DB_RECNO, db.DB_CREATE)
        for x in letters:
            recno = d.append(x * 60)
            if not type(recno) == type(0):
                raise AssertionError
            if not recno >= 1:
                raise AssertionError
            if verbose:
                print recno,
                continue
        
        if verbose:
            print 
        
        stat = d.stat()
        if verbose:
            pprint(stat)
        
        for recno in range(1, len(d) + 1):
            data = d[recno]
            if verbose:
                print data
            
            if not type(data) == type(''):
                raise AssertionError
            if not data == d.get(recno):
                raise AssertionError
        
        
        try:
            data = d[0]
        except db.DBInvalidArgError:
            val = None
            if not val[0] == db.EINVAL:
                raise AssertionError
            if verbose:
                print val
            
        except:
            verbose

        self.fail('expected exception')
        
        try:
            data = d[100]
        except KeyError:
            pass

        self.fail('expected exception')
        data = d.get(100)
        if not data == None:
            raise AssertionError
        keys = d.keys()
        if verbose:
            print keys
        
        if not type(keys) == type([]):
            raise AssertionError
        if not type(keys[0]) == type(123):
            raise AssertionError
        if not len(keys) == len(d):
            raise AssertionError
        items = d.items()
        if verbose:
            pprint(items)
        
        if not type(items) == type([]):
            raise AssertionError
        if not type(items[0]) == type(()):
            raise AssertionError
        if not len(items[0]) == 2:
            raise AssertionError
        if not type(items[0][0]) == type(123):
            raise AssertionError
        if not type(items[0][1]) == type(''):
            raise AssertionError
        if not len(items) == len(d):
            raise AssertionError
        if not d.has_key(25):
            raise AssertionError
        del d[25]
        if not not d.has_key(25):
            raise AssertionError
        d.delete(13)
        if not not d.has_key(13):
            raise AssertionError
        data = d.get_both(26, 'z' * 60)
        if not data == 'z' * 60:
            raise AssertionError
        if verbose:
            print data
        
        fd = d.fd()
        if verbose:
            print fd
        
        c = d.cursor()
        rec = c.first()
        while rec:
            if verbose:
                print rec
            
            rec = c.next()
        c.set(50)
        rec = c.current()
        if verbose:
            print rec
        
        c.put(-1, 'a replacement record', db.DB_CURRENT)
        c.set(50)
        rec = c.current()
        if not rec == (50, 'a replacement record'):
            raise AssertionError
        if verbose:
            print rec
        
        rec = c.set_range(30)
        if verbose:
            print rec
        
        c.close()
        d.close()
        d = db.DB()
        d.open(self.filename)
        c = d.cursor()
        d[100] = 'way out there'
        if not d[100] == 'way out there':
            raise AssertionError
        
        try:
            data = d[99]
        except KeyError:
            pass

        self.fail('expected exception')
        
        try:
            d.get(99)
        except db.DBKeyEmptyError:
            val = None
            if not val[0] == db.DB_KEYEMPTY:
                raise AssertionError
            if verbose:
                print val
            
        except:
            verbose

        self.fail('expected exception')
        rec = c.set(40)
        while rec:
            if verbose:
                print rec
            
            rec = c.next()
        c.close()
        d.close()

    
    def test02_WithSource(self):
        '''
        A Recno file that is given a "backing source file" is essentially a
        simple ASCII file.  Normally each record is delimited by 
 and so is
        just a line in the file, but you can set a different record delimiter
        if needed.
        '''
        source = os.path.join(os.path.dirname(sys.argv[0]), 'db_home/test_recno.txt')
        f = open(source, 'w')
        f.close()
        d = db.DB()
        d.set_re_delim(10)
        d.set_re_delim('\n')
        d.set_re_source(source)
        d.open(self.filename, db.DB_RECNO, db.DB_CREATE)
        data = 'The quick brown fox jumped over the lazy dog'.split()
        for datum in data:
            d.append(datum)
        
        d.sync()
        d.close()
        text = open(source, 'r').read()
        text = text.strip()
        if verbose:
            print text
            print data
            print text.split('\n')
        
        if not text.split('\n') == data:
            raise AssertionError
        d = db.DB()
        d.set_re_source(source)
        d.open(self.filename, db.DB_RECNO)
        d[3] = 'reddish-brown'
        d[8] = 'comatose'
        d.sync()
        d.close()
        text = open(source, 'r').read()
        text = text.strip()
        if verbose:
            print text
            print text.split('\n')
        
        if not text.split('\n') == 'The quick reddish-brown fox jumped over the comatose dog'.split():
            raise AssertionError

    
    def test03_FixedLength(self):
        d = db.DB()
        d.set_re_len(40)
        d.set_re_pad('-')
        d.set_re_pad(45)
        d.open(self.filename, db.DB_RECNO, db.DB_CREATE)
        for x in letters:
            d.append(x * 35)
        
        d.append('.' * 40)
        
        try:
            d.append('bad' * 20)
        except db.DBInvalidArgError:
            val = None
            if not val[0] == db.EINVAL:
                raise AssertionError
            if verbose:
                print val
            
        except:
            verbose

        self.fail('expected exception')
        c = d.cursor()
        rec = c.first()
        while rec:
            if verbose:
                print rec
            
            rec = c.next()
        c.close()
        d.close()



def test_suite():
    return unittest.makeSuite(SimpleRecnoTestCase)

if __name__ == '__main__':
    unittest.main(defaultTest = 'test_suite')

