# Source Generated with Decompyle++
# File: in.pyo (Python 2.3)

'''TestCases for exercising a Recno DB.
'''
import os
import sys
import errno
import tempfile
from pprint import pprint
import unittest
from test_all import verbose

try:
    from bsddb import db
except ImportError:
    from bsddb3 import db

letters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'

class SimpleRecnoTestCase(unittest.TestCase):
    
    def setUp(self):
        self.filename = tempfile.mktemp()

    
    def tearDown(self):
        
        try:
            os.remove(self.filename)
        except OSError:
            e = None
            if e.errno != errno.EEXIST:
                raise 
            
        except:
            e.errno != errno.EEXIST


    
    def test01_basic(self):
        d = db.DB()
        d.open(self.filename, db.DB_RECNO, db.DB_CREATE)
        for x in letters:
            recno = d.append(x * 60)
            if verbose:
                print recno,
                continue
        
        if verbose:
            print 
        
        stat = d.stat()
        if verbose:
            pprint(stat)
        
        for recno in range(1, len(d) + 1):
            data = d[recno]
            if verbose:
                print data
                continue
        
        
        try:
            data = d[0]
        except db.DBInvalidArgError:
            val = None
            if verbose:
                print val
            
        except:
            verbose

        self.fail('expected exception')
        
        try:
            data = d[100]
        except KeyError:
            pass

        self.fail('expected exception')
        data = d.get(100)
        keys = d.keys()
        if verbose:
            print keys
        
        items = d.items()
        if verbose:
            pprint(items)
        
        del d[25]
        d.delete(13)
        data = d.get_both(26, 'z' * 60)
        if verbose:
            print data
        
        fd = d.fd()
        if verbose:
            print fd
        
        c = d.cursor()
        rec = c.first()
        while rec:
            if verbose:
                print rec
            
            rec = c.next()
        c.set(50)
        rec = c.current()
        if verbose:
            print rec
        
        c.put(-1, 'a replacement record', db.DB_CURRENT)
        c.set(50)
        rec = c.current()
        if verbose:
            print rec
        
        rec = c.set_range(30)
        if verbose:
            print rec
        
        c.close()
        d.close()
        d = db.DB()
        d.open(self.filename)
        c = d.cursor()
        d[100] = 'way out there'
        
        try:
            data = d[99]
        except KeyError:
            pass

        self.fail('expected exception')
        
        try:
            d.get(99)
        except db.DBKeyEmptyError:
            val = None
            if verbose:
                print val
            
        except:
            verbose

        self.fail('expected exception')
        rec = c.set(40)
        while rec:
            if verbose:
                print rec
            
            rec = c.next()
        c.close()
        d.close()

    
    def test02_WithSource(self):
        '''
        A Recno file that is given a "backing source file" is essentially a
        simple ASCII file.  Normally each record is delimited by 
 and so is
        just a line in the file, but you can set a different record delimiter
        if needed.
        '''
        source = os.path.join(os.path.dirname(sys.argv[0]), 'db_home/test_recno.txt')
        f = open(source, 'w')
        f.close()
        d = db.DB()
        d.set_re_delim(10)
        d.set_re_delim('\n')
        d.set_re_source(source)
        d.open(self.filename, db.DB_RECNO, db.DB_CREATE)
        data = 'The quick brown fox jumped over the lazy dog'.split()
        for datum in data:
            d.append(datum)
        
        d.sync()
        d.close()
        text = open(source, 'r').read()
        text = text.strip()
        if verbose:
            print text
            print data
            print text.split('\n')
        
        d = db.DB()
        d.set_re_source(source)
        d.open(self.filename, db.DB_RECNO)
        d[3] = 'reddish-brown'
        d[8] = 'comatose'
        d.sync()
        d.close()
        text = open(source, 'r').read()
        text = text.strip()
        if verbose:
            print text
            print text.split('\n')
        

    
    def test03_FixedLength(self):
        d = db.DB()
        d.set_re_len(40)
        d.set_re_pad('-')
        d.set_re_pad(45)
        d.open(self.filename, db.DB_RECNO, db.DB_CREATE)
        for x in letters:
            d.append(x * 35)
        
        d.append('.' * 40)
        
        try:
            d.append('bad' * 20)
        except db.DBInvalidArgError:
            val = None
            if verbose:
                print val
            
        except:
            verbose

        self.fail('expected exception')
        c = d.cursor()
        rec = c.first()
        while rec:
            if verbose:
                print rec
            
            rec = c.next()
        c.close()
        d.close()



def test_suite():
    return unittest.makeSuite(SimpleRecnoTestCase)

if __name__ == '__main__':
    unittest.main(defaultTest = 'test_suite')

