# Source Generated with Decompyle++
# File: in.pyo (Python 2.3)

'''A parser of RFC 2822 and MIME email messages.
'''
import re
from cStringIO import StringIO
from types import ListType
from email import Errors
from email import Message
EMPTYSTRING = ''
NL = '\n'

try:
    (True, False)
except NameError:
    True = 1
    False = 0

NLCRE = re.compile('\r\n|\r|\n')

class Parser:
    
    def __init__(self, _class = Message.Message, strict = False):
        """Parser of RFC 2822 and MIME email messages.

        Creates an in-memory object tree representing the email message, which
        can then be manipulated and turned over to a Generator to return the
        textual representation of the message.

        The string must be formatted as a block of RFC 2822 headers and header
        continuation lines, optionally preceeded by a `Unix-from' header.  The
        header block is terminated either by the end of the string or by a
        blank line.

        _class is the class to instantiate for new message objects when they
        must be created.  This class must have a constructor that can take
        zero arguments.  Default is Message.Message.

        Optional strict tells the parser to be strictly RFC compliant or to be
        more forgiving in parsing of ill-formatted MIME documents.  When
        non-strict mode is used, the parser will try to make up for missing or
        erroneous boundaries and other peculiarities seen in the wild.
        Default is non-strict parsing.
        """
        self._class = _class
        self._strict = strict

    
    def parse(self, fp, headersonly = False):
        '''Create a message structure from the data in a file.

        Reads all the data from the file and returns the root of the message
        structure.  Optional headersonly is a flag specifying whether to stop
        parsing after reading the headers or not.  The default is False,
        meaning it parses the entire contents of the file.
        '''
        root = self._class()
        firstbodyline = self._parseheaders(root, fp)
        if not headersonly:
            self._parsebody(root, fp, firstbodyline)
        
        return root

    
    def parsestr(self, text, headersonly = False):
        '''Create a message structure from a string.

        Returns the root of the message structure.  Optional headersonly is a
        flag specifying whether to stop parsing after reading the headers or
        not.  The default is False, meaning it parses the entire contents of
        the file.
        '''
        return self.parse(StringIO(text), headersonly = headersonly)

    
    def _parseheaders(self, container, fp):
        lastheader = ''
        lastvalue = []
        lineno = 0
        firstbodyline = None
        while True:
            line = fp.readline()
            if not line:
                break
            
            line = line.splitlines()[0]
            if not line:
                break
            
            lineno += 1
            if line.startswith('From '):
                if lineno == 1:
                    container.set_unixfrom(line)
                    continue
                elif self._strict:
                    raise Errors.HeaderParseError('Unix-from in headers after first rfc822 header')
                
            
            if line[0] in ' \t':
                if not lastheader:
                    raise Errors.HeaderParseError('Continuation line seen before first header')
                
                lastvalue.append(line)
                continue
            
            i = line.find(':')
            if i < 0:
                if self._strict:
                    raise Errors.HeaderParseError("Not a header, not a continuation: ``%s''" % line)
                elif lineno == 1 and line.startswith('--'):
                    continue
                else:
                    firstbodyline = line
                    break
            
            if lastheader:
                container[lastheader] = NL.join(lastvalue)
            
            lastheader = line[:i]
            lastvalue = [
                line[i + 1:].lstrip()]
        if lastheader:
            container[lastheader] = NL.join(lastvalue)
        
        return firstbodyline

    
    def _parsebody(self, container, fp, firstbodyline = None):
        boundary = container.get_boundary()
        isdigest = container.get_content_type() == 'multipart/digest'
        if boundary:
            preamble = epilogue = None
            separator = '--' + boundary
            payload = fp.read()
            if firstbodyline is not None:
                payload = firstbodyline + '\n' + payload
            
            mo = re.search('(?P<sep>' + re.escape(separator) + ')(?P<ws>[ \\t]*)', payload)
            if not mo:
                if self._strict:
                    raise Errors.BoundaryError("Couldn't find starting boundary: %s" % boundary)
                
                container.set_payload(payload)
                return None
            
            start = mo.start()
            if start > 0:
                preamble = payload[0:start]
            
            start += len(mo.group('sep')) + len(mo.group('ws'))
            mo = NLCRE.search(payload, start)
            if mo:
                start += len(mo.group(0))
            
            cre = re.compile('(?P<sep>\r\n|\r|\n)' + re.escape(separator) + '--')
            mo = cre.search(payload, start)
            if mo:
                terminator = mo.start()
                linesep = mo.group('sep')
                if mo.end() < len(payload):
                    epilogue = payload[mo.end():]
                
            elif self._strict:
                raise Errors.BoundaryError("Couldn't find terminating boundary: %s" % boundary)
            else:
                mo = re.search('(?P<sep>\r\n|\r|\n){2}$', payload)
                if not mo:
                    mo = re.search('(?P<sep>\r\n|\r|\n)$', payload)
                    if not mo:
                        raise Errors.BoundaryError('No terminating boundary and no trailing empty line')
                    
                
                linesep = mo.group('sep')
                terminator = len(payload)
            parts = re.split(linesep + re.escape(separator) + '[ \\t]*' + linesep, payload[start:terminator])
            for part in parts:
                if isdigest:
                    if part.startswith(linesep):
                        msgobj = self._class()
                        part = part[len(linesep):]
                    else:
                        (parthdrs, part) = part.split(linesep + linesep, 1)
                        msgobj = self.parsestr(parthdrs, headersonly = 1)
                    msgobj.set_default_type('message/rfc822')
                    maintype = msgobj.get_content_maintype()
                    if maintype in ('message', 'multipart'):
                        submsgobj = self.parsestr(part)
                        msgobj.attach(submsgobj)
                    else:
                        msgobj.set_payload(part)
                else:
                    msgobj = self.parsestr(part)
                container.preamble = preamble
                container.epilogue = epilogue
                container.attach(msgobj)
            
        elif container.get_main_type() == 'multipart':
            raise Errors.BoundaryError('multipart message with no defined boundary')
        elif container.get_type() == 'message/delivery-status':
            blocks = []
            while True:
                blockmsg = self._class()
                self._parseheaders(blockmsg, fp)
                if not len(blockmsg):
                    break
                
                blocks.append(blockmsg)
            container.set_payload(blocks)
        elif container.get_main_type() == 'message':
            
            try:
                msg = self.parse(fp)
            except Errors.HeaderParseError:
                msg = self._class()
                self._parsebody(msg, fp)

            container.attach(msg)
        else:
            text = fp.read()
            if firstbodyline is not None:
                text = firstbodyline + '\n' + text
            
            container.set_payload(text)



class HeaderParser(Parser):
    """A subclass of Parser, this one only meaningfully parses message headers.

    This class can be used if all you're interested in is the headers of a
    message.  While it consumes the message body, it does not parse it, but
    simply makes it available as a string payload.

    Parsing with this subclass can be considerably faster if all you're
    interested in is the message headers.
    """
    
    def _parsebody(self, container, fp, firstbodyline = None):
        text = fp.read()
        if firstbodyline is not None:
            text = firstbodyline + '\n' + text
        
        container.set_payload(text)


