# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

'''
pluginbuilder.py -- Tools to assemble MacOS X python-based plugin bundles 

This module defines one class to build python based plugin bundles for MacOS X.
Examples of plugin bundles include PreferencePanes and InterfaceBuilder 
palletes.

The PluginBuilder class is instantated with a number of keyword arguments and
have a build() method that will do all the work. See the class docstrings for
a description of the constructor arguments.

The module contains a main program that can be used in two ways:
  % python pluginbuilder.py [options] build
  % python buildplugin.py [options] build

Where "buildplugin.py" is a user-supplied setup.py-like script following this
model:

  from PyObjCTools.pluginbuilder import buildplugin

  buildplugin(<lots-of-keyword-args>)

'''
__all__ = ('BundleBuilderError', 'buildplugin', 'PluginBuilder')
import bundlebuilder
import os
import getopt
import sys
from bundlebuilder import BundleBuilderError
CODE = '/*\n * !!!! THIS IS GENERATED CODE, DO NOT EDIT !!!!\n */\n#import <Foundation/Foundation.h>\n\n#include <Python/Python.h>\n\n@interface PyObjC_Bundle_%(BUNDLE)s\n{\n}\n\n+(void)load;\n\n@end\n\n@implementation PyObjC_Bundle_%(BUNDLE)s\n\n+(void)load\n{\n\tNSBundle* bundle;\n\tNSString* mainPath;\n\tFILE*     fp;\n        PyObject *m, *d, *v;\n\n        //NSLog(@"Loading prefpane %(MAINFILE)s %(BUNDLE)s");\n\n\tbundle = [NSBundle bundleForClass:self];\n\t[bundle load];\n\n        mainPath = [bundle pathForResource:@"%(MAINFILE)s" ofType:@"py"];\n\n\tif (mainPath == NULL) {\n\t\tmainPath = [bundle pathForResource:@"%(MAINFILE)s" ofType:@"pyc"];\n\t\t[NSException raise:NSInternalInconsistencyException \n\t\t\tformat:@"Bundle %%@ does not contain a %(MAINFILE)s file",\n\t\t\tbundle];\n\t}\n\n\tif (!Py_IsInitialized()) {\n\t\tPy_Initialize();\n\t}\n\n\tfp = fopen([mainPath cString], "r");\n\tif (fp == NULL) {\n\t\t[NSException raise:NSInternalInconsistencyException \n\t\t\tformat:@"Cannot open %(MAINFILE)s in bundle %%@",\n\t\t\tbundle];\n\t}\n\n        /*\n         * We cannot use \'PyRun_SimpleFile\' because that would make it \n         * too easy to cause interference between two Python based plugins.\n         *\n         * This code works like PyRun_SimpleFile, but runs the code in\n         * a unique module (we hope), that has a __path__ attribute. This way\n         * we don\'t have to update sys.path, and that will hopefully avoid\n         * most interference between plugins.\n         *\n\t * PyRun_SimpleFile(fp, [mainPath cString]);\n         */\n        m = PyImport_AddModule("__main_%(BUNDLE)s__");\n        if (m == NULL) {\n                PyErr_Print();\n\t\t[NSException raise:NSInternalInconsistencyException \n\t\t\tformat:@"Cannot create main module for bundle %%@",\n\t\t\tbundle];\n\t}\n\n        PyModule_AddStringConstant(m, "__file__", (char*)[mainPath cString]);\n        PyModule_AddStringConstant(m, "__path__", \n                    (char*)[[bundle resourcePath] cString]);\n        d = PyModule_GetDict(m);\n        if (d == NULL) {\n                PyErr_Print();\n\t\t[NSException raise:NSInternalInconsistencyException \n\t\t\tformat:@"Failed to initialize bundle %%@",\n\t\t\tbundle];\n        }\n\n        if (PyDict_GetItemString(d, "__builtins__") == NULL) {\n            PyObject* bimod = PyImport_ImportModule("__builtin__");\n            if (bimod == NULL || PyDict_SetItemString(d, "__builtins__", bimod) != 0) {\n                PyErr_Print();\n\t\t[NSException raise:NSInternalInconsistencyException \n\t\t\tformat:@"Failed to initialize bundle %%@",\n\t\t\tbundle];\n            }\n            Py_XDECREF(bimod);\n        }\n\n        if (PyErr_Occurred()) {\n                PyErr_Print();\n\t\t[NSException raise:NSInternalInconsistencyException \n\t\t\tformat:@"Failed to initialize bundle %%@",\n\t\t\tbundle];\n        }\n        v = PyRun_File(fp, [mainPath cString], Py_file_input, d, d);\n        if (v == NULL) {\n            PyErr_Print();\n\t    [NSException raise:NSInternalInconsistencyException \n\t\tformat:@"Failed to initialize bundle %%@",\n\t\tbundle];\n        }\n        Py_DECREF(v);\n        if (Py_FlushLine()) {\n            PyErr_Clear();\n        }\n}\n\n@end\n'

class PluginBuilder(bundlebuilder.BundleBuilder):
    type = 'BNDL'
    platform = 'MacOS'
    mainmodule = None
    nibname = None
    principalClass = None
    iconfile = None
    bundlesuffix = '.bnld'
    standalone = 0
    strip = 0
    pymodules = []
    missingModules = []
    maybeMissingModules = []
    binaries = []
    
    def setup(self):
        if self.mainmodule is None:
            raise bundlebuilder.BundleBuilderError, "must specify 'mainmodule' when building a plugin bundle"
        else:
            self.files.append((self.mainmodule, os.path.join('Contents', 'Resources', self.mainmodule)))
        if self.standalone:
            raise bundlebuilder.BundleBuilderError, "can't specify 'standalone' at the moment, sorry"
        
        if self.strip:
            raise bundlebuilder.BundleBuilderError, "can't specify 'strip' at the moment, sorry"
        
        if self.nibname is None:
            raise bundlebuilder.BundleBuilderError, "must specify 'nibname' when building a plugin bundle"
        
        self.execdir = os.path.join('Contents', self.platform)
        if self.name is None:
            self.name = os.path.splitext(self.mainmodule)[0]
        
        if not self.name.endswith(self.bundlesuffix):
            self.name += self.bundlesuffix
        
        if self.nibname:
            self.plist.NSMainNibFile = self.nibname
            if not hasattr(self.plist, 'NSPrincipalClass'):
                if self.principalClass is None:
                    raise bundlebuilder.BundleBuilderError, "must specify 'principalClass' when building a plugin bundle"
                
                self.plist.NSPrincipalClass = self.principalClass
            
        
        bundlebuilder.BundleBuilder.setup(self)
        self.plist.CFBundleExecutable = self.name
        if self.standalone:
            self.findDependencies()
        

    
    def preProcess(self):
        if self.iconfile is not None:
            iconbase = os.path.basename(self.iconfile)
            self.plist.CFBunelIconFile = iconbase
            self.files.append((self.iconfile, os.path.join(resdir, iconbase)))
        

    
    def postProcess(self):
        destdir = os.path.join(self.bundlepath, self.execdir)
        if not os.path.exists(destdir):
            os.mkdir(destdir)
        
        compileBundle(os.path.join(destdir, self.name), self.name, self.mainmodule)
        self.binaries.append(os.path.join(self.execdir, self.name))
        if self.standalone:
            self.addPythonModules()
        
        if self.strip:
            self.stripBinaries()
        
        if self.missingModules or self.maybeMissingModules:
            self.reportMissing()
        



def compileBundle(outputName, bundle, mainModule):
    tmpFile = outputName + '.m'
    mainModule = os.path.splitext(mainModule)[0]
    fd = open(tmpFile, 'w')
    
    try:
        fd.write(CODE % {
            'BUNDLE': bundle,
            'MAINFILE': mainModule })
        fd.close()
        
        quoteArg = lambda arg: arg.replace("'", '\'"\'"\'')
        fd = os.popen("cc -bundle -Wno-long-double -o '%s' '%s' -framework Foundation -framework Python 2>&1" % (quoteArg(outputName), quoteArg(tmpFile)), 'r')
        for ln in fd:
            sys.stdout.write(ln)
        
        xit = fd.close()
        if xit is not None:
            raise ValueError, 'Compilation failed'
    finally:
        os.unlink(tmpFile)


cmdline_doc = 'Usage:\n  python pluginbuilder.py [options] command\n  python mybuildscript.py [options] command\n\nCommands:\n  build      build the application\n  report     print a report\n\nOptions:\n  -b, --builddir=DIR     the build directory; defaults to "build"\n  -n, --name=NAME        plugin name\n  -r, --resource=FILE    extra file or folder to be copied to Resources\n  -f, --file=SRC:DST     extra file or folder to be copied into the bundle;\n                         DST must be a path relative to the bundle root\n  -m, --mainmodule=FILE the Python main program\n  -p, --plist=FILE       .plist file (default: generate one)\n      --nib=NAME         main nib name\n      --principalClass=NAME principal class name\n      --iconfile=FILE    filename of the icon (an .icns file) to be used\n                         as the Finder icon\n      --standalone       build a standalone application, which is fully\n                         independent of a Python installation\n      --lib=FILE         shared library or framework to be copied into\n                         the bundle\n  -x, --exclude=MODULE   exclude module (with --standalone)\n  -i, --include=MODULE   include module (with --standalone)\n      --package=PACKAGE  include a whole package (with --standalone)\n      --strip            strip binaries (remove debug info)\n  -v, --verbose          increase verbosity level\n  -q, --quiet            decrease verbosity level\n  -h, --help             print this message\n'

def usage(msg = None):
    if msg:
        print msg
    
    print cmdline_doc
    sys.exit(1)


def main(builder = None):
    if builder is None:
        builder = PluginBuilder(verbosity = 1)
    
    shortopts = 'b:n:r:f:m:p:x:i:hvq'
    longopts = ('builddir=', 'name=', 'resource=', 'file=', 'mainmodule=', 'creator=', 'nib=', 'principalClas=', 'plist=', 'help', 'verbose', 'quiet', 'standalone', 'exclude=', 'include=', 'package=', 'strip', 'iconfile=', 'lib=')
    
    try:
        (options, args) = getopt.getopt(sys.argv[1:], shortopts, longopts)
    except getopt.error:
        usage()

    for opt, arg in options:
        if opt in ('-b', '--builddir'):
            builder.builddir = arg
            continue
        if opt in ('-n', '--name'):
            builder.name = arg
            continue
        if opt in ('-r', '--resource'):
            builder.resources.append(arg)
            continue
        if opt in ('-f', '--file'):
            srcdst = arg.split(':')
            if len(srcdst) != 2:
                usage('-f or --file argument must be two paths, separated by a colon')
            
            builder.files.append(srcdst)
            continue
        if opt == '--iconfile':
            builder.iconfile = arg
            continue
        if opt == '--lib':
            builder.libs.append(arg)
            continue
        if opt == '--nib':
            builder.nibname = arg
            continue
        if opt in ('-p', '--plist'):
            builder.plist = Plist.fromFile(arg)
            continue
        if opt in ('-h', '--help'):
            usage()
            continue
        if opt in ('-v', '--verbose'):
            builder.verbosity += 1
            continue
        builder
        if opt in ('-q', '--quiet'):
            builder.verbosity -= 1
            continue
        builder
        if opt == '--standalone':
            builder.standalone = 1
            continue
        if opt in ('-x', '--exclude'):
            builder.excludeModules.append(arg)
            continue
        if opt in ('-i', '--include'):
            builder.includeModules.append(arg)
            continue
        if opt == '--package':
            builder.includePackages.append(arg)
            continue
        if opt == '--strip':
            builder.strip = 1
            continue
        if opt == '--principalClass':
            builder.principalClass = arg
            continue
    
    if len(args) != 1:
        usage("Must specify one command ('build', 'report' or 'help')")
    
    command = args[0]
    if command == 'build':
        builder.setup()
        builder.build()
    elif command == 'report':
        builder.setup()
        builder.report()
    elif command == 'help':
        usage()
    else:
        usage("Unknown command '%s'" % command)


def buildplugin(**kwargs):
    builder = PluginBuilder(**kwargs)
    main(builder)

if __name__ == '__main__':
    main()

