# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

from cStringIO import StringIO
from array import array
import fieldinfos
import fieldswriter
import term
import terminfoswriter
import terminfo
import lupy.search.similarity as similarity

class DocumentWriter(object):
    
    def __init__(self, directory, analyzer, mfl):
        self.directory = directory
        self.analyzer = analyzer
        self.maxFieldLength = mfl
        self.postingTable = { }
        self.termBuffer = term.Term('', '')

    
    def addDocument(self, segment, doc):
        fi = self.fieldInfos = fieldinfos.FieldInfos()
        fi.add(doc)
        fi.writeDir(self.directory, segment + '.fnm')
        fieldsWriter = fieldswriter.FieldsWriter(self.directory, segment, self.fieldInfos)
        
        try:
            fieldsWriter.addDocument(doc)
        finally:
            fieldsWriter.close()

        self.postingTable = { }
        self.fieldLengths = [
            0] * len(self.fieldInfos)
        self.invertDocument(doc)
        postings = self.sortPostingTable()
        self.writePostings(postings, segment)
        self.writeNorms(doc, segment)

    
    def invertDocument(self, doc):
        fields = doc.fields()
        for field in doc.fields():
            fieldName = field.name()
            fieldNumber = self.fieldInfos.fieldNumber(fieldName)
            position = self.fieldLengths[fieldNumber]
            if field.isIndexed:
                if not (field.isTokenized):
                    self.addPosition(fieldName, field.stringValue(), position)
                    position += 1
                elif field.readerValue() is not None:
                    reader = field.readerValue()
                elif field.stringValue() is not None:
                    reader = StringIO(field.stringValue())
                else:
                    raise Exception, 'Field must have either a String or Reader value'
                stream = self.analyzer.tokenStream(fieldName, reader)
                
                try:
                    t = stream.next()
                    while t is not None:
                        self.addPosition(fieldName, t.termText(), position)
                        position += 1
                        if position > self.maxFieldLength:
                            break
                        
                        t = stream.next()
                finally:
                    stream.close()

            
            self.fieldLengths[fieldNumber] = position
        

    
    def addPosition(self, field, text, position):
        self.termBuffer.set(field, text)
        ti = self.postingTable.get(self.termBuffer, None)
        if ti is not None:
            freq = ti.freq
            ti.positions.append(position)
            ti.freq = freq + 1
        else:
            trm = term.Term(field, text, False)
            self.postingTable[trm] = Posting(trm, position)

    
    def sortPostingTable(self):
        arr = self.postingTable.values()
        arr.sort()
        return arr

    
    def writePostings(self, postings, segment):
        freq = None
        prox = None
        tis = None
        
        try:
            freq = self.directory.createFile(segment + '.frq')
            prox = self.directory.createFile(segment + '.prx')
            tis = terminfoswriter.TermInfosWriter(self.directory, segment, self.fieldInfos)
            ti = terminfo.TermInfo()
            for posting in postings:
                ti.set(1, freq.getFilePointer(), prox.getFilePointer())
                tis.add(posting.term, ti)
                f = posting.freq
                if f == 1:
                    freq.writeVInt(1)
                else:
                    freq.writeVInt(0)
                    freq.writeVInt(f)
                lastPosition = 0
                positions = posting.positions
                for position in positions:
                    prox.writeVInt(position - lastPosition)
                    lastPosition = position
                
        finally:
            if freq is not None:
                freq.close()
            
            if prox is not None:
                prox.close()
            
            if tis is not None:
                tis.close()
            


    
    def writeNorms(self, doc, segment):
        for field in doc.fields():
            if field.isIndexed:
                fieldNumber = self.fieldInfos.fieldNumber(field.name())
                norm = self.directory.createFile(segment + '.f' + str(fieldNumber))
                
                try:
                    norm.writeByte(similarity.normInt(self.fieldLengths[fieldNumber]))
                finally:
                    norm.close()

                continue
        



class Posting(object):
    __slots__ = [
        'term',
        'freq',
        'positions']
    
    def __init__(self, t, position):
        self.term = t
        self.freq = 1
        self.positions = array('i', [
            1])
        self.positions[0] = position

    
    def __repr__(self):
        s = '<Posting:'
        s += str(self.term) + '>'
        return s

    
    def __cmp__(self, other):
        return cmp(self.term, other.term)


