# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

import sys
from lupy.store import ramdirectory
from lupy.store import fsdirectory
import documentwriter
import segmentinfos
import segmentinfo
import segmentmerger
import segmentreader

class IndexWriter(object):
    __slots__ = [
        'analyzer',
        'directory',
        'infoStream',
        'maxFieldLength',
        'maxMergeDocs',
        'mergeFactor',
        'ramDirectory',
        'segmentInfos',
        'writeLock']
    
    def __init__(self, path, a, create = False):
        if path is None:
            if create is True:
                self.directory = ramdirectory.RAMDirectory()
            else:
                self.directory = path
        else:
            self.directory = fsdirectory.FSDirectory(path, create)
        self.analyzer = a
        self.infoStream = None
        self.maxFieldLength = 10000
        self.maxMergeDocs = sys.maxint
        self.mergeFactor = 20
        self.segmentInfos = segmentinfos.SegmentInfos()
        self.ramDirectory = ramdirectory.RAMDirectory()
        if create is True:
            self.segmentInfos.write(self.directory)
        else:
            self.segmentInfos.read(self.directory)

    
    def close(self):
        self.flushRamSegments()
        self.ramDirectory.close()
        self.directory.close()

    
    def docCount(self):
        count = 0
        for si in self.segmentInfos:
            count += si.docCount
        
        return count

    
    def addDocument(self, doc):
        dw = documentwriter.DocumentWriter(self.ramDirectory, self.analyzer, self.maxFieldLength)
        segmentName = self.newSegmentName()
        dw.addDocument(segmentName, doc)
        self.segmentInfos.append(segmentinfo.SegmentInfo(segmentName, 1, self.ramDirectory))
        self.maybeMergeSegments()

    
    def newSegmentName(self):
        res = '_' + str(self.segmentInfos.counter)
        self.segmentInfos.counter += 1
        return res

    
    def optimize(self):
        self.flushRamSegments()
        while len(self.segmentInfos) > 1 and len(self.segmentInfos) == 1 and segmentreader.SegmentReader.hasDeletions(self.segmentInfos[0]) or self.segmentInfos[0].dir != self.directory:
            minSegment = len(self.segmentInfos) - self.mergeFactor
            if minSegment < 0:
                self.mergeSegments(0)
                continue
            self.mergeSegments(minSegment)

    
    def addIndexes(self, dirs):
        '''Merges all segments from an array of indexes into this index.
        
        This may be used to parallelize batch indexing.  A large document
        collection can be broken into sub-collections.  Each sub-collection can be
        indexed in parallel, on a different thread, process or machine.  The
        complete index can then be created by merging sub-collection indexes
        with this method.
        
        After this completes, the index is optimized.'''
        self.optimize()
        for d in dirs:
            sis = segmentinfos.SegmentInfos()
            sis.read(d)
            for j in range(len(sis)):
                self.segmentInfos.append(sis[j])
            
        
        self.optimize()

    
    def flushRamSegments(self):
        '''Merges all RAM-resident segments'''
        sis = self.segmentInfos
        minSegment = len(sis) - 1
        docCount = 0
        while minSegment >= 0 and sis[minSegment].dir == self.ramDirectory:
            docCount += sis[minSegment].docCount
            minSegment -= 1
        if minSegment < 0 and docCount + sis[minSegment].docCount > self.mergeFactor or not (sis[len(sis) - 1].dir == self.ramDirectory):
            minSegment += 1
        
        if minSegment >= len(sis):
            return None
        
        self.mergeSegments(minSegment)

    
    def maybeMergeSegments(self):
        '''Incremental segment merger'''
        targetMergeDocs = self.mergeFactor
        while targetMergeDocs <= self.maxMergeDocs:
            minSegment = len(self.segmentInfos)
            mergeDocs = 0
            minSegment -= 1
            while minSegment >= 0:
                si = self.segmentInfos[minSegment]
                if si.docCount >= targetMergeDocs:
                    break
                
                mergeDocs += si.docCount
                minSegment -= 1
            if mergeDocs >= targetMergeDocs:
                self.mergeSegments(minSegment + 1)
            else:
                break
            targetMergeDocs *= self.mergeFactor

    
    def mergeSegments(self, minSegment):
        '''Pops segments off of segmentInfos stack down to minSegment,
        merges them, and pushes the merged index onto the top of the
        segmentInfos stack'''
        mergedName = self.newSegmentName()
        mergedDocCount = 0
        merger = segmentmerger.SegmentMerger(self.directory, mergedName)
        segmentsToDelete = []
        for i in range(minSegment, len(self.segmentInfos)):
            si = self.segmentInfos[i]
            reader = segmentreader.SegmentReader(si)
            merger.add(reader)
            if reader.directory is self.directory or reader.directory is self.ramDirectory:
                segmentsToDelete.append(reader)
            
            mergedDocCount += si.docCount
        
        merger.merge()
        self.segmentInfos = self.segmentInfos[:minSegment]
        self.segmentInfos.append(segmentinfo.SegmentInfo(mergedName, mergedDocCount, self.directory))
        self.segmentInfos.write(self.directory)
        self.deleteSegments(segmentsToDelete)

    
    def deleteSegments(self, segs):
        """Some operating systems (e.g. Windows) don't permit a file to be deleted
        while it is opened for read (e.g. by another process or thread).  So we
        assume that when a delete fails it is because the file is open in another
        process, and queue the file for subsequent deletion."""
        deletable = []
        self.deleteFilesList(self.readDeleteableFiles(), deletable)
        for reader in segs:
            if reader.directory is self.directory:
                self.deleteFilesList(reader.files(), deletable)
            else:
                self.deleteFilesDir(reader.files(), reader.directory)
            self.writeDeleteableFiles(deletable)
        

    
    def deleteFilesDir(self, files, dir):
        for file in files:
            dir.deleteFile(file)
        

    
    def deleteFilesList(self, files, deletable):
        for file in files:
            
            try:
                self.directory.deleteFile(file)
            continue
            except OSError:
                if self.directory.fileExists(file):
                    deletable.append(file)
                
                self.directory.fileExists(file)
            

        

    
    def readDeleteableFiles(self):
        result = []
        if not self.directory.fileExists('deletable'):
            return result
        
        input = self.directory.openFile('deletable')
        
        try:
            i = input.readInt()
            while i > 0:
                result.append(input.readString())
                i -= 1
        finally:
            input.close()

        return result

    
    def writeDeleteableFiles(self, files):
        output = self.directory.createFile('deletable.new')
        
        try:
            output.writeInt(len(files))
            for file in files:
                output.writeString(file)
        finally:
            output.close()

        self.directory.renameFile('deletable.new', 'deletable')


