# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

import sys
from array import array
import lupy.store.ramdirectory as ramdirectory
import lupy.store.fsdirectory as fsdirectory
import lupy.util.bitvector as bitvector
from indexreader import IndexReader
import fieldinfos
import terminfo
import terminfosreader
import terminfoswriter
import fieldsreader
import fieldswriter
import segmenttermdocs
import segmenttermpositions
import segmentmergequeue
import segmentmergeinfo

class SegmentMerger(object):
    __slots__ = [
        'directory',
        'fieldInfos',
        'freqOutput',
        'proxOutput',
        'queue',
        'readers',
        'segment',
        'termInfo',
        'termInfosWriter']
    
    def __init__(self, dir, name):
        self.directory = dir
        self.segment = name
        self.queue = None
        self.freqOutput = None
        self.proxOutput = None
        self.termInfosWriter = None
        self.readers = []
        self.termInfo = terminfo.TermInfo()

    
    def add(self, reader):
        self.readers.append(reader)

    
    def appendPostings(self, smis, n):
        lastDoc = 0
        df = 0
        for i in range(n):
            smi = smis[i]
            postings = smi.postings
            base = smi.base
            docMap = smi.docMap
            smi.termEnum.termInfo(self.termInfo)
            postings.seekTi(self.termInfo)
            while postings.next():
                if docMap is None:
                    doc = base + postings.doc()
                else:
                    doc = base + docMap[postings.doc()]
                if doc < lastDoc:
                    raise Exception, 'docs out of order'
                
                docCode = doc - lastDoc << 1
                lastDoc = doc
                freq = postings.freq()
                if freq == 1:
                    self.freqOutput.writeVInt(docCode | 1)
                else:
                    self.freqOutput.writeVInt(docCode)
                    self.freqOutput.writeVInt(freq)
                lastPosition = 0
                for j in range(freq):
                    position = postings.nextPosition()
                    self.proxOutput.writeVInt(position - lastPosition)
                    lastPosition = position
                
                df += 1
        
        return df

    
    def merge(self):
        
        try:
            self.mergeFields()
            self.mergeTerms()
            self.mergeNorms()
        finally:
            for reader in self.readers:
                reader.close()
            


    
    def mergeFields(self):
        self.fieldInfos = fieldinfos.FieldInfos()
        for reader in self.readers:
            self.fieldInfos.addFieldInfos(reader.fieldInfos)
        
        self.fieldInfos.writeDir(self.directory, self.segment + '.fnm')
        fieldsWriter = fieldswriter.FieldsWriter(self.directory, self.segment, self.fieldInfos)
        
        try:
            for reader in self.readers:
                deletedDocs = reader.deletedDocs
                maxDoc = reader.maxDoc()
                for j in range(maxDoc):
                    if deletedDocs is None or not deletedDocs.get(j):
                        fieldsWriter.addDocument(reader.document(j))
                        continue
                
        finally:
            fieldsWriter.close()


    
    def mergeNorms(self):
        for i in range(len(self.fieldInfos)):
            fi = self.fieldInfos.fieldInfoInt(i)
            if fi.isIndexed:
                output = self.directory.createFile(self.segment + '.f' + str(i))
                
                try:
                    for reader in self.readers:
                        deletedDocs = reader.deletedDocs
                        input = reader.normStream(fi.name)
                        maxDoc = reader.maxDoc()
                        
                        try:
                            for k in range(maxDoc):
                                if input is None:
                                    norm = 0
                                else:
                                    norm = input.readByte()
                                output.writeByte(norm)
                        finally:
                            if input is not None:
                                input.close()
                            

                finally:
                    output.close()

                continue
        

    
    def mergeTermInfo(self, smis, n):
        freqPointer = self.freqOutput.getFilePointer()
        proxPointer = self.proxOutput.getFilePointer()
        df = self.appendPostings(smis, n)
        if df > 0:
            self.termInfo.set(df, freqPointer, proxPointer)
            self.termInfosWriter.add(smis[0].term, self.termInfo)
        

    
    def mergeTermInfos(self):
        queue = segmentmergequeue.SegmentMergeQueue(len(self.readers))
        base = 0
        for reader in self.readers:
            termEnum = reader.terms()
            smi = segmentmergeinfo.SegmentMergeInfo(base, termEnum, reader)
            base += reader.numDocs()
            if smi.next():
                queue.put(smi)
                continue
            smi.close()
        
        match = [
            0] * len(self.readers)
        while queue.size() > 0:
            matchSize = 0
            match[matchSize] = queue.pop()
            matchSize += 1
            term = match[0].term
            top = queue.top()
            while top is not None and cmp(term, top.term) == 0:
                match[matchSize] = queue.pop()
                matchSize += 1
                top = queue.top()
            self.mergeTermInfo(match, matchSize)
            while matchSize > 0:
                matchSize -= 1
                smi = match[matchSize]
                if smi.next():
                    queue.put(smi)
                    continue
                smi.close()

    
    def mergeTerms(self):
        
        try:
            self.freqOutput = self.directory.createFile(self.segment + '.frq')
            self.proxOutput = self.directory.createFile(self.segment + '.prx')
            self.termInfosWriter = terminfoswriter.TermInfosWriter(self.directory, self.segment, self.fieldInfos)
            self.mergeTermInfos()
        finally:
            if self.freqOutput is not None:
                self.freqOutput.close()
            
            if self.proxOutput is not None:
                self.proxOutput.close()
            
            if self.termInfosWriter is not None:
                self.termInfosWriter.close()
            
            if self.queue is not None:
                self.queue.close()
            


    
    def segementReader(self, i):
        return self.readers[i]


