# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

'''A ranked list of documents, used to hold search results.
            '''
import hitdoc

class Hits(object):
    __slots__ = [
        'filter',
        'first',
        'hitDocs',
        'last',
        'length',
        'maxDocs',
        'numDocs',
        'query',
        'searcher']
    
    def __init__(self, s, q, f):
        '''Initialize scoreDocs and totalHits.
        '''
        self.query = q
        self.searcher = s
        self.filter = f
        self.hitDocs = []
        self.maxDocs = 200
        self.numDocs = 0
        self.first = None
        self.length = 0
        self.getMoreDocs(50)

    
    def __len__(self):
        return self.length

    
    def addToFront(self, hitDoc):
        '''Insert at front of cache.
        '''
        if self.first is None:
            self.last = hitDoc
        else:
            self.first.prev = hitDoc
        hitDoc.next = self.first
        self.first = hitDoc
        hitDoc.prev = None
        self.numDocs += 1

    
    def __getitem__(self, index):
        return self.doc(index)

    
    def doc(self, n):
        hitDoc = self.hitDoc(n)
        self.remove(hitDoc)
        self.addToFront(hitDoc)
        if self.numDocs > self.maxDocs:
            oldLast = self.last
            self.remove(self.last)
            oldLast.doc = None
        
        if hitDoc.doc is None:
            hitDoc.doc = self.searcher.doc(hitDoc.id)
        
        return hitDoc.doc

    
    def getMoreDocs(self, min):
        '''Tries to add new documents to hitDocs.
            Ensures that the hit numbered B{min} has been retrieved.
            '''
        if len(self.hitDocs) > min:
            min = len(self.hitDocs)
        
        n = min * 2
        topDocs = self.searcher.searchQueryFilterInt(self.query, self.filter, n)
        self.length = topDocs.totalHits
        scoreDocs = topDocs.scoreDocs
        scoreNorm = 1.0
        if self.length > 0 and scoreDocs[0].score > 1.0:
            scoreNorm = 1.0 / scoreDocs[0].score
        
        if len(scoreDocs) < self.length:
            end = len(scoreDocs)
        else:
            end = self.length
        for i in range(len(self.hitDocs), end, 1):
            hd = hitdoc.HitDoc(scoreDocs[i].score * scoreNorm, scoreDocs[i].doc)
            self.hitDocs.append(hd)
        

    
    def hitDoc(self, n):
        if n >= self.length:
            raise IndexError, 'Not a valid hit number ' + str(n)
        
        if n >= len(self.hitDocs):
            self.getMoreDocs(n)
        
        return self.hitDocs[n]

    
    def remove(self, hitDoc):
        if hitDoc.doc is None:
            return None
        
        if hitDoc.next is None:
            last = hitDoc.prev
        else:
            hitDoc.next.prev = hitDoc.prev
        if hitDoc.prev is None:
            self.first = hitDoc.next
        else:
            hitDoc.prev.next = hitDoc.next
        self.numDocs -= 1

    
    def score(self, n):
        ''' Returns the score for the nth document in the set.
        '''
        return self.hitDoc(n).score

    
    def __repr__(self):
        s = '<' + str(len(self)) + ' Hit'
        if len(self) == 1:
            s += '>'
        else:
            s += 's>'
        return s


