# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

'''The abstract base class for search implementations.

          Subclasses implement search over a single index, over
          multiple indices, and over indices on remote servers.'''
import similarity
import searcher
import hits
import query as querymodule
import hitqueue
import hitcollector
import scoredoc
import topdocs
import lupy.index.indexreader as indexreader

class IndexSearcher(searcher.Searcher):
    __slots__ = [
        'reader']
    
    def __init__(self, dirOrPath):
        '''Creates a searcher searching the provided index.
        '''
        if type(dirOrPath) == type(''):
            self.reader = indexreader.IndexReader.open(dirOrPath)
        else:
            self.reader = indexreader.IndexReader.openDir(dirOrPath)

    
    def setReader(self, r):
        self.reader = r

    
    def close(self):
        '''Frees resources associated with this Searcher.'''
        self.reader.close()

    
    def doc(self, i):
        '''For use by L{HitCollector} implementations.'''
        return self.reader.document(i)

    
    def docFreq(self, term):
        return self.reader.docFreq(term)

    
    def maxDoc(self):
        return self.reader.maxDoc()

    
    def searchQueryFilterHitColl(self, query, filter, results):
        '''Lower-level search API.

        L{HitCollector#collect(int,float)} is called for every
        non-zero scoring document.

        Applications should only use this if they need <i>all</i> of the
        matching documents.  The high-level search API
        ({L{Searcher#search(Query)}) is usually more efficient, as it skips
        non-high-scoring hits.

        - B{query} to match documents
        - B{filter} if non-null, a bitset used to eliminate some documents
        - B{results} to receive hits
        '''
        collector = results
        if filter is not None:
            bits = filter.bits(reader)
            collector = hitcollector.HitCollector()
        
        scorer = query.Query.scorer(query, self, self.reader)
        if scorer is None:
            return None
        
        scorer.score(collector, self.reader.maxDoc())

    
    def searchQueryFilterInt(self, query, filter, nDocs):
        scorer = querymodule.Query.getScorer(query, self, self.reader)
        if scorer is None:
            return topdocs.TopDocs(0, [])
        
        if filter is not None:
            bits = filter.bits(reader)
        else:
            bits = None
        hq = hitqueue.HitQueue(nDocs)
        totalHits = [
            0]
        minScore = 0.0
        hc = hitcollector.HitCollector()
        hc.minScore = minScore
        
        def _cbHitCollect(doc, scr):
            if scr > 0.0 and bits is None or bits.get(doc):
                totalHits[0] += 1
                if scr >= hc.minScore:
                    hq.put(scoredoc.ScoreDoc(doc, scr))
                    if hq.size() > nDocs:
                        hq.pop()
                        hc.minScore = hq.top().score
                    
                
            

        hc.collect = _cbHitCollect
        scorer.score(hc, self.reader.maxDoc())
        scoreDocs = [
            0] * hq.size()
        for i in range(hq.size() - 1, -1, -1):
            scoreDocs[i] = hq.pop()
        
        return topdocs.TopDocs(totalHits[0], scoreDocs)

    
    def fieldNames(self):
        return self.reader.fieldNames()


