# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

'''A Query that matches documents containing a particular
            sequence of terms. This may be combined with other terms
            with a L{BooleanQuery}.
            '''
import query
import termscorer
import similarity
import exactphrasescorer

class PhraseQuery(query.Query):
    __slots__ = [
        'field',
        'idf',
        'slop',
        'terms',
        'weight']
    
    def __init__(self):
        '''Constructs an empty phrase query.'''
        query.Query.__init__(self)
        self.idf = 0.0
        self.slop = 0
        self.terms = []
        self.weight = 0.0

    
    def add(self, term):
        '''Adds a term to the end of the query phrase.'''
        if len(self.terms) == 0:
            self.field = term.field()
        elif term.field() != self.field:
            raise Exception, 'All phrase terms must be in the same field: ' + str(term)
        
        self.terms.append(term)

    
    def getSlop(self):
        '''Returns the slop.  See setSlop().'''
        return self.slop

    
    def normalize(self, norm):
        self.weight *= norm
        self.weight *= self.idf

    
    def scorer(self, reader):
        if len(self.terms) == 0:
            return None
        
        if len(self.terms) == 1:
            term = self.terms[0]
            docs = reader.termDocsTerm(term)
            if docs is None:
                return None
            
            return termscorer.TermScorer(docs, reader.norms(term.field()), weight)
        
        tps = []
        for term in self.terms:
            p = reader.termPositionsTerm(term)
            if p is None:
                return None
            
            tps.append(p)
        
        if self.slop == 0:
            return exactphrasescorer.ExactPhraseScorer(tps, reader.normsField(self.field), self.weight)
        else:
            return sloppyphrasescorer.SloppyPhraseScorer(tps, reader.norms(self.field), self.weight)

    
    def setSlop(self, s):
        '''Sets the number of other words permitted between words in query phrase.
        If zero, then this is an exact phrase search.  For larger values this works
        like a <code>WITHIN</code> or <code>NEAR</code> operator.

        <p>The slop is in fact an edit-distance, where the units correspond to
        moves of terms in the query phrase out of position.  For example, to switch
        the order of two words requires two moves (the first move places the words
        atop one another), so to permit re-orderings of phrases, the slop must be
        at least two.

        <p>More exact matches are scored higher than sloppier matches, thus search
        results are sorted by exactness.

        <p>The slop is zero by default, requiring exact matches.'''
        self.slop = s

    
    def sumOfSquaredWeights(self, searcher):
        for term in self.terms:
            self.idf += similarity.idfTerm(term, searcher)
        
        self.weight = self.idf * self.boost
        return self.weight * self.weight

    
    def toString(self, f):
        '''Prints a user-readable version of this query'''
        buffer = ''
        if not (self.field == f):
            buffer += f + ':'
        
        buffer += '\\'
        for i in range(len(self.terms)):
            term = self.terms[i]
            buffer += term.text()
            if i != len(self.terms) - 1:
                buffer += ' '
                continue
        
        buffer += '\\'
        if self.slop != 0:
            buffer += '~' + str(self.slop)
        
        if self.boost != 1.0:
            buffer += '^' + str(self.boost)
        
        return buffer


