# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

'''Cross-platform (posix/nt) API for flock-style file locking.

Synopsis:

   import portalocker
   file = open("somefile", "r+")
   portalocker.lock(file, portalocker.LOCK_EX)
   file.seek(12)
   file.write("foo")
   file.close()

If you know what you\'re doing, you may choose to

   portalocker.unlock(file)

before closing the file, but why?

Methods:

   lock( file, flags )
   unlock( file )

Constants:

   LOCK_EX
   LOCK_SH
   LOCK_NB

I learned the win32 technique for locking files from sample code
provided by John Nielsen <nielsenjf@my-deja.com> in the documentation
that accompanies the win32 modules.

Author: Jonathan Feinberg <jdf@pobox.com>
Version: $Id: locker.py,v 1.1 2003/10/17 16:15:29 amir Exp $
'''
import os
importSuccess = True
if os.name == 'nt':
    
    try:
        import win32con
        import win32file
        import pywintypes
    except ImportError:
        importSuccess = False
        print 'Missing windows library win32con. Locking disabled!!!'

    if importSuccess:
        LOCK_EX = win32con.LOCKFILE_EXCLUSIVE_LOCK
        LOCK_SH = 0
        LOCK_NB = win32con.LOCKFILE_FAIL_IMMEDIATELY
        __overlapped = pywintypes.OVERLAPPED()
    else:
        LOCK_EX = LOCK_SH = LOCK_NB = None
elif os.name == 'posix':
    import fcntl
    LOCK_EX = fcntl.LOCK_EX
    LOCK_SH = fcntl.LOCK_SH
    LOCK_NB = fcntl.LOCK_NB
else:
    raise RuntimeError('PortaLocker only defined for nt and posix platforms')
if os.name == 'nt':
    if importSuccess:
        
        def lock(file, flags):
            hfile = win32file._get_osfhandle(file.fileno())
            win32file.LockFileEx(hfile, flags, 0, -65536, __overlapped)

        
        def unlock(file):
            hfile = win32file._get_osfhandle(file.fileno())
            win32file.UnlockFileEx(hfile, 0, -65536, __overlapped)

    else:
        
        def lock(file, flags):
            pass

        
        def unlock(file):
            pass

elif os.name == 'posix':
    
    def lock(file, flags):
        fcntl.flock(file.fileno(), flags)

    
    def unlock(file):
        fcntl.flock(file.fileno(), fcntl.LOCK_UN)


