# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

import os
import time
import glob
import sys
import weakref
import threading
import urlparse
import cgi
from ypage.compiler import PageCompiler, CompilerError
from snakelet import Snakelet
PAGE_SOURCE_OUTPUTDIR = 'tmp'

class YpageEngine:
    import string
    ALLOWED_MODULE_CHARS = string.letters + string.digits + '_'
    MODULE_PREFIX = 'Ypage_'
    
    def __init__(self, name, vhost, writePageSource):
        self.cache = { }
        self.name = name
        self.writePageSource = writePageSource
        if self.writePageSource:
            self.pageSourceOutputDir = os.path.join(PAGE_SOURCE_OUTPUTDIR, vhost)
            
            try:
                os.mkdir(self.pageSourceOutputDir)
            except OSError:
                pass

            print 'Erasing old page sources from', self.pageSourceOutputDir
            prefix = self.createPageModuleName('') + '*'
            for f in glob.glob(os.path.join(self.pageSourceOutputDir, prefix)):
                os.remove(f)
            
        
        self.lock = threading.Lock()

    
    def clearCache(self):
        self.lock.acquire()
        
        try:
            self.cache.clear()
        finally:
            self.lock.release()


    
    def loadPage(self, pageFile, pageName, webapp, pageTemplateArgs = None):
        (_ignore, _ignore, pageName, pageArgs, _ignore) = urlparse.urlsplit(pageName)
        if pageArgs:
            pageFile = webapp.getFullPath(pageName)
            pageArgs = dict(cgi.parse_qsl(pageArgs))
            if not pageTemplateArgs:
                pass
            pageArgs.update({ })
        else:
            pageArgs = pageTemplateArgs
        pageModule = self.createPageModuleName(pageName)
        
        try:
            
            try:
                self.lock.acquire()
                mustCompile = True
                if pageModule in self.cache:
                    compiledPageModule = self.cache[pageModule]
                    mustCompile = os.path.getmtime(pageFile) >= compiledPageModule.__mtime__
                
                if mustCompile:
                    import time
                    compiledPageModule = self.compilePageModule(pageFile, pageName, pageModule, webapp.getDefaultOutputEncoding(), webapp.getDefaultPageTemplate())
                    compiledPageModule.__mtime__ = time.time()
                    self.cache[pageModule] = compiledPageModule
            finally:
                self.lock.release()

            if not pageArgs:
                pass
            return compiledPageModule.Page(pageName, weakref.ref(webapp), { })
        except CompilerError:
            x = None
            xmsg = str(x)
            del x
            raise CompilerError(xmsg)


    
    def compilePageModule(self, pageFile, pageName, pageModule, defaultOutputEncoding = None, defaultPageTemplate = None):
        starttime = time.clock()
        print 'parsing ypage "%s"... ' % pageName,
        pythonSource = PageCompiler().compilePage(pageFile, defaultOutputEncoding, defaultPageTemplate)
        print 'compiling...',
        bytecode = compile(pythonSource, '<generated_code>', 'exec')
        import imp
        module = imp.new_module(pageModule)
        exec bytecode in module.__dict__
        print 'done (%d ms)' % (time.clock() - starttime) * 1000
        if self.writePageSource:
            sourcefile = os.path.join(self.pageSourceOutputDir, pageModule + '.py')
            f = open(sourcefile, 'wb')
            f.write(pythonSource)
            f.close()
            print 'Page source written to', sourcefile
        
        return module

    
    def runPage(self, page, _request, _response, defaultOutputEncoding):
        import cStringIO
        output = cStringIO.StringIO()
        if not page.getPageEncoding():
            pass
        outputEncoding = defaultOutputEncoding
        if outputEncoding:
            if sys.version_info[:2] >= (2, 3):
                errs = 'xmlcharrefreplace'
            else:
                errs = 'replace'
            import codecs
            output = codecs.getwriter(outputEncoding)(output, errors = errs)
            _response.content_encoding = outputEncoding
        
        oldOut = _response.YswitchOutput(output)
        page.createOutput(output, _request, _response)
        _response.YswitchOutput(oldOut)
        size = len(output.getvalue())
        output.seek(0)
        return (output, size, outputEncoding, page.getPageContentTypeAndDisposition())

    
    def createPageModuleName(self, pageName):
        pageName = pageName.replace('/', '__').replace('\\', '__')
        result = [
            self.name + '_']
        for c in pageName:
            if c in self.ALLOWED_MODULE_CHARS:
                result.append(c)
                continue
            result.append('_')
        
        return self.MODULE_PREFIX + ''.join(result)

    
    def addPageVars(self, page, webapp, request, response):
        page.Request = request
        page.RequestCtx = request.getContext()
        page.ApplicationCtx = webapp.context
        page.WebApp = weakref.proxy(webapp)
        page.URLprefix = webapp.getURLprefix()
        if page.requiresSession() != Snakelet.SESSION_NOT_NEEDED:
            session = webapp.addSessionCookie(request, response)
            if session:
                page.SessionCtx = session.getContext()
                page.User = session.getLoggedInUser()
            
        

    
    def copyPageVars(clazz, sourcePage, destPage):
        destPage.Request = sourcePage.Request
        destPage.RequestCtx = sourcePage.RequestCtx
        destPage.ApplicationCtx = sourcePage.ApplicationCtx
        destPage.WebApp = sourcePage.WebApp
        destPage.URLprefix = sourcePage.URLprefix
        destPage.write = sourcePage.write
        destPage.out = sourcePage.out
        if hasattr(sourcePage, 'SessionCtx'):
            destPage.SessionCtx = sourcePage.SessionCtx
        
        if hasattr(sourcePage, 'User'):
            destPage.User = sourcePage.User
        

    copyPageVars = classmethod(copyPageVars)


class Ypage(Snakelet):
    
    class PageAbortError(Exception):
        pass

    
    class PageSendErrorError(Exception):
        
        def __init__(self, error, msg = None):
            self.error = error
            self.msg = msg


    
    class PageRedirectURL(Exception):
        
        def __init__(self, url):
            self.url = url


    
    class PageHTTPRedirectURL(Exception):
        
        def __init__(self, url):
            self.url = url


    
    def __init__(self, pageName, webappref, pageArgs = None):
        Snakelet.__init__(self, pageName, webappref)
        if not pageArgs:
            pass
        self.PageArgs = { }

    
    def getDescription(self):
        return 'Ypage snakelet'

    
    def getPageEncoding(self):
        return None

    
    def getPageContentTypeAndDisposition(self):
        return (None, None)

    
    def createOutput(self, out, _request, _response):
        self.out = out
        if self.getPageEncoding():
            self.write = self._writeUnicode
        else:
            self.write = self._writeString
        
        try:
            self.getWebApp().addPageHeaders(self, _response)
            self.create(out, _request, _response)
        except Ypage.PageAbortError:
            pax = None
            self.out.write('<hr><strong>Page aborted<p>' + str(pax) + '</strong>')
        except Ypage.PageSendErrorError:
            pex = None
            _response.sendError(pex.error, pex.msg)
            self.out.truncate(0)
            return None
        except Ypage.PageHTTPRedirectURL:
            prx = None
            self.out.truncate(0)
            _response.HTTPredirect(prx.url)
            return None
        except Ypage.PageRedirectURL:
            prx = None
            self.out.truncate(0)
            self.redirect(prx.url, _request, _response)
            return None
        except Exception:
            x = None
            self.getWebApp().reportSnakeletException(self, x, None, self.out, _request, _response, self.getErrorPage())
        finally:
            sys.exc_clear()
            del self.write

        self.out.flush()

    
    def _writeUnicode(self, strng):
        self.out.write(unicode(strng))

    
    def _writeString(self, strng):
        self.out.write(str(strng))

    
    def abort(self, message = ''):
        raise Ypage.PageAbortError(message)

    
    def Yredirect(self, URL):
        raise Ypage.PageRedirectURL(URL)

    
    def Yhttpredirect(self, URL):
        raise Ypage.PageHTTPRedirectURL(URL)

    
    def sendError(self, error, msg = None):
        raise Ypage.PageSendErrorError(error, msg)

    
    def copyPageVars(self, source, dest):
        YpageEngine.copyPageVars(source, dest)


