# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

from util import ContextContainer
import time
import weakref
import threading

class SecurityError(Exception):
    
    def __init__(self, value, extrainfo = None):
        Exception.__init__(self, value)
        if extrainfo:
            self.Snakelets_extrainfo = extrainfo
        



class SessionAddressError(SecurityError):
    pass


class Session:
    
    def __init__(self, sessid, timeoutsecs, remoteAddr, webappref):
        self.sessionID = sessid
        self.context = ContextContainer()
        self.timeoutsecs = timeoutsecs
        self.touch()
        self.new = True
        self.user = None
        self.callbacks = []
        self.createtime = time.time()
        if type(webappref) != weakref.ReferenceType:
            raise TypeError('webapp for session must be a weakref')
        
        self.webappref = webappref
        self.remoteAddr = remoteAddr
        self.lock = threading.Lock()

    
    def setRequestData(self, request, response):
        if self.remoteAddr != str(request.getRealRemoteAddr()):
            print 'SECURITY ERROR: attempt to transport session to a different remote address'
            print 'From:', self.remoteAddr, '  to:', request.getRealRemoteAddr()
            (new, old) = (request.getRealRemoteAddr(), self.remoteAddr)
            del request
            del response
            raise SessionAddressError('attempt to transport session to a different remote address', 'The last request accessing your session information came from a different IP address (%s) than before (%s). This is a security error, to prevent somebody else from stealing your data,' % (new, old))
        
        self.requestref = weakref.ref(request)
        self.responseref = weakref.ref(response)

    
    def getID(self):
        return self.sessionID

    
    def isNew(self):
        return self.new

    
    def getLoggedInUser(self):
        return self.user

    
    def loginUser(self, user):
        self.user = user

    
    def getContext(self):
        return self.context

    
    def getRemoteAddr(self):
        return self.remoteAddr

    
    def touch(self):
        self.lastused = time.time()
        self.new = False

    
    def isOverAged(self):
        return time.time() - self.lastused > self.timeoutsecs

    
    def destroy(self, external = False):
        self.lock.acquire()
        
        try:
            for cb in self.callbacks:
                cb(self, self.webappref())
            
            if not external and self.webappref():
                self.webappref()._deleteSession(self, self.responseref())
            
            if self.requestref():
                self.requestref().setSession(None)
        finally:
            self.lock.release()


    
    def destroyIfOverAged(self):
        if self.isOverAged():
            self.destroy()
        

    
    def registerCallback(self, callback):
        
        try:
            self.callbacks.remove(callback)
        except ValueError:
            pass

        self.callbacks.append(callback)


