# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

import parser
from tokenizer import TokenizerError
import sys
linesep = '\n'
CREATE_METHOD = 'create'
TEMPLATED_CREATE_METHOD = 'createTemplated'

class CompilerError(Exception):
    pass


class PageCompiler:
    
    def __init__(self):
        pass

    
    def compilePage(self, filename, defaultOutputEncoding = None, defaultPageTemplate = None):
        self.declarations = { }
        self.imports = []
        self.gobbleWS = True
        self.outputEncoding = None
        self.contentType = None
        self.contentDisposition = None
        self.allowCaching = None
        self.indentChars = '\t'
        self.session = 'yes'
        self.signinpage = None
        self.errorpage = None
        self.pagemethod = CREATE_METHOD
        self.baseclasses = []
        self.pageTemplate = defaultPageTemplate
        self.pageTemplateArgs = None
        self.filename = filename
        (self.inputEncoding, self.outputEncoding) = parser.determineInputOutputEncodings(filename)
        if not (self.outputEncoding):
            self.outputEncoding = defaultOutputEncoding
        
        
        try:
            
            try:
                pageSourceStream = None
                if self.inputEncoding:
                    import codecs
                    pageSourceStream = codecs.open(filename, mode = 'rb', encoding = self.inputEncoding)
                else:
                    pageSourceStream = file(filename, 'r')
                syntaxtree = parser.Parser(pageSourceStream, filename, self.inputEncoding).parse()
            finally:
                if pageSourceStream:
                    pageSourceStream.close()
                
                del pageSourceStream

            (result, indent) = self.generateCode(syntaxtree, 0)
            return result
        except (TokenizerError, parser.ParserError):
            px = None
            sys.exc_clear()
            raise CompilerError(px)


    
    def _addTextBlock(self, astlist, child):
        if len(astlist) > 0 and isinstance(astlist[-1], parser.TextBlock):
            astlist[-1].text += child.text
        else:
            astlist.append(child)

    
    def optimize(self, ast):
        optimized = []
        prev = None
        for child in ast:
            if isinstance(child, parser.Script):
                optimized.append(parser.Script(child.text, self.optimize(child)))
            elif isinstance(child, parser.Comment):
                optimized.append(child)
            elif isinstance(child, parser.Whitespace):
                if not (self.gobbleWS):
                    self._addTextBlock(optimized, child)
                
            elif isinstance(child, parser.TextBlock):
                if self.gobbleWS and not isinstance(prev, parser.Expression):
                    newTxt = child.text.lstrip()
                    if newTxt:
                        self._addTextBlock(optimized, parser.TextBlock(newTxt))
                    
                else:
                    self._addTextBlock(optimized, child)
            else:
                optimized.append(child)
            prev = child
        
        return optimized

    
    def processDeclarations(self, syntaxtree):
        gatheredWhitespace = ''
        while isinstance(syntaxtree[0], parser.Declaration) or isinstance(syntaxtree[0], parser.Whitespace):
            c = syntaxtree[0]
            del syntaxtree[0]
            if isinstance(c, parser.Whitespace):
                gatheredWhitespace += c.text
                continue
            
            c.name = c.name.lower()
            if c.name == 'import':
                if c.value.split()[0] in ('import', 'from'):
                    self.imports.append(c.value)
                else:
                    raise CompilerError("invalid import statement: '" + c.value + "'")
            c.value.split()[0] in ('import', 'from')
            if c.name == 'gobblews':
                if c.value.lower() in ('yes', 'no'):
                    self.gobbleWS = c.value.lower() == 'yes'
                else:
                    raise CompilerError('gobblews must be yes or no')
            c.value.lower() in ('yes', 'no')
            if c.name == 'allowcaching':
                if c.value.lower() in ('yes', 'no'):
                    self.allowCaching = c.value.lower() == 'yes'
                else:
                    raise CompilerError('allowcaching must be yes or no')
            c.value.lower() in ('yes', 'no')
            if c.name == 'session':
                value = c.value.lower()
                if ';' in value:
                    (value, self.signinpage) = value.split(';', 1)
                
                if value in ('yes', 'no', 'user', 'valid'):
                    self.session = value
                    if self.signinpage and self.session != 'user':
                        print 'IGNORING SIGNIN PAGE DUE TO NON-USER SESSION'
                    
                else:
                    raise CompilerError('session must be yes, no, valid or user')
            value in ('yes', 'no', 'user', 'valid')
            if c.name == 'outputencoding':
                if parser.stringInQuotes(c.value):
                    c.value = c.value[1:-1]
                
                if self.outputEncoding != c.value:
                    raise RuntimeError('output encoding was parsed wrongly')
                
            self.outputEncoding != c.value
            if c.name == 'inputencoding':
                if parser.stringInQuotes(c.value):
                    c.value = c.value[1:-1]
                
                if self.inputEncoding != c.value:
                    raise RuntimeError('input encoding was parsed wrongly')
                
            self.inputEncoding != c.value
            if c.name == 'contenttype':
                self.contentType = c.value
                continue
            if c.name == 'disposition':
                self.contentDisposition = c.value
                continue
            if c.name == 'indent':
                v = c.value.lower()
                if v == '8spaces':
                    self.indentChars = ' ' * 8
                
                if v == '4spaces':
                    self.indentChars = ' ' * 4
                elif v == 'tab':
                    self.indentChars = '\t'
                else:
                    raise CompilerError('indent type must be tab or 4spaces or 8spaces')
            v == '4spaces'
            if c.name == 'errorpage':
                if parser.stringInQuotes(c.value):
                    c.value = c.value[1:-1]
                
                self.errorpage = c.value
                continue
            if c.name == 'pagemethod':
                self.pagemethod = c.value.strip()
                continue
            if c.name == 'inherit':
                self.baseclasses = [ clz.strip() for clz in c.value.split(',') ]
                continue
            []
            if c.name == 'pagetemplate':
                if parser.stringInQuotes(c.value):
                    c.value = c.value[1:-1]
                
                page = c.value.strip()
                if page.lower() == 'none':
                    page = None
                
                self.pageTemplate = page
                continue
            if c.name == 'pagetemplateargs':
                import cgi
                if parser.stringInQuotes(c.value):
                    c.value = c.value[1:-1]
                
                self.pageTemplateArgs = dict(cgi.parse_qsl(c.value))
                continue
            raise CompilerError('unknown declaration: %s=%s' % (c.name, c.value))
        return ''

    
    def generateCode(self, c, blockindent):
        indent = self.indentChars * blockindent
        code = u''
        if isinstance(c, parser.Document):
            gatheredWhitespace = self.processDeclarations(c)
            if gatheredWhitespace and not (self.gobbleWS):
                c.insert(0, parser.Whitespace(gatheredWhitespace))
            
            c = self.optimize(c)
            code += 'from snakeserver.YpageEngine import Ypage' + linesep
            if self.session == 'no':
                session = 'SESSION_NOT_NEEDED'
            elif self.session == 'yes':
                session = 'SESSION_WANTED'
            elif self.session == 'valid':
                session = 'SESSION_REQUIRED'
            elif self.session == 'user':
                session = 'SESSION_LOGIN_REQUIRED'
            else:
                raise CompilerError('invalid session selector: ' + self.session)
            code += self.indentChars + 'def requiresSession(self): return self.' + session + linesep
            if self.session == 'user' and self.signinpage:
                code += self.indentChars + "def getSessionSigninPage(self): return '" + self.signinpage + "'" + linesep
            
            if self.outputEncoding:
                code += self.indentChars + 'def getPageEncoding(self): return %r' % self.outputEncoding + linesep
            
            if self.allowCaching:
                code += self.indentChars + 'def allowCaching(self): return %r' % self.allowCaching + linesep
            
            if self.contentType or self.contentDisposition:
                code += self.indentChars + 'def getPageContentTypeAndDisposition(self): return %r,%r' % (self.contentType, self.contentDisposition) + linesep
            
            if self.pageTemplate:
                if self.filename.endswith(self.pageTemplate):
                    self.pageTemplate = None
                else:
                    self.pagemethod = TEMPLATED_CREATE_METHOD
                    code += self.indentChars + 'def %s(self,out,_request,_response):' % CREATE_METHOD + linesep
                    code += self.indentChars * 2 + 'tplpath="%s"' % self.pageTemplate + linesep
                    code += self.indentChars * 2 + 'fulltplpath=self.WebApp.getFullPath(tplpath)' + linesep
                    if self.pageTemplateArgs:
                        pageTemplateArgs = ',%s' % repr(self.pageTemplateArgs)
                    else:
                        pageTemplateArgs = ''
                    code += self.indentChars * 2 + 'tplpage=self.getWebApp().getPageEngine().loadPage(fulltplpath,tplpath,self.getWebApp()' + pageTemplateArgs + ')' + linesep
                    code += self.indentChars * 2 + 'self.copyPageVars(self,tplpage)' + linesep
                    code += self.indentChars * 2 + 'return tplpage.%s(out, _request, _response, templatedPage=self)' % CREATE_METHOD + linesep
            
            code += self.indentChars + 'def ' + self.pagemethod + '(self,out,_request,_response,templatedPage=None):' + linesep
            for imp in self.imports:
                code += self.indentChars * 2 + imp + linesep
            
            code += self.indentChars * 2 + '_w=out.write' + linesep
            if self.errorpage:
                code += self.indentChars * 2 + "self.setErrorPage('" + self.errorpage + "')\n"
            
            for child in c:
                (subcode, indent) = self.generateCode(child, blockindent + 2)
                code += subcode
                if indent > 0:
                    indent -= 3
                
                blockindent = indent
            
        elif isinstance(c, parser.Declaration):
            if c.name == 'import':
                if c.value.split()[0] in ('import', 'from'):
                    self.imports.append(c.value)
                else:
                    raise CompilerError("invalid import statement: '" + c.value + "'")
            
            self.declarations[c.name] = c.value
        elif isinstance(c, parser.Comment):
            code += indent + '# ' + c.text + linesep
        elif isinstance(c, parser.URLCall):
            code += indent + 'self.include(%s, _request, _response)' % c.url + linesep
        elif isinstance(c, parser.URLForward):
            code += indent + 'self.Yredirect(%s)' % c.url + linesep
        elif isinstance(c, parser.HTTPRedirect):
            code += indent + 'self.Yhttpredirect(%s)' % c.url + linesep
        elif isinstance(c, parser.TextBlock):
            text = repr(c.text)
            if isinstance(c, parser.Whitespace) and self.gobbleWS:
                return (code, 0)
            
            code += indent + '_w(' + text + ')' + linesep
        elif isinstance(c, parser.Expression):
            if self.outputEncoding:
                code += indent + '_w(unicode(' + c.text + '))' + linesep
            else:
                code += indent + '_w(str(' + c.text + '))' + linesep
        elif isinstance(c, parser.Script):
            lines = [ indent + line for line in c.text.split('\n') ]
            code += linesep.join(lines) + linesep
            newblockindent = blockindent + 1
            for child in c:
                (subcode, indent) = self.generateCode(child, newblockindent)
                code += subcode
            
            return (code, newblockindent)
        elif isinstance(c, parser.InsertPageBody):
            code += indent + 'if templatedPage: templatedPage.%s(out, _request, _response)' % TEMPLATED_CREATE_METHOD + linesep
            code += indent + "else: _w('{no templated page available}')" + linesep
        else:
            raise CompilerError('invalid syntax element ' + repr(c))
        return (code, 0)

    
    def determineIndent(self, lastline, blockindent):
        indentchars = len(self.indentChars) * blockindent
        lastline = lastline[indentchars:]
        extraindent = 0
        while lastline.startswith(self.indentChars):
            lastline = lastline[len(self.indentChars):]
            extraindent += 1
        return blockindent + extraindent



def main(args):
    compiler = PageCompiler()
    if len(args) != 3:
        print 'usage: python ' + args[0] + ' page.y output'
        print 'where output is a file or - for stdout'
        raise SystemExit
    
    if args[2] == '-':
        output = sys.stdout
    else:
        output = file(args[2], 'wb')
    result = compiler.compilePage(args[1])
    output.write(result.encode('iso-8859-1', 'replace'))

if __name__ == '__main__':
    import sys
    main(sys.argv)

