# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

import sre
import tokenizer
import types
import os
import codecs

class ParserError(Exception):
    
    def __init__(self, msg, linenum = None):
        if linenum:
            msg = msg + ' @' + str(linenum)
        
        Exception.__init__(self, msg)



def isWhitespace(text):
    if not type(text) in types.StringTypes and len(text) == 0:
        pass
    return text.isspace()


def stringInQuotes(txt):
    if txt and len(txt) >= 2:
        if txt[0] in ("'", '"') and txt[-1] in ("'", '"'):
            return True
        
        if txt[0] in ("'", '"') or txt[-1] in ("'", '"'):
            raise ParserError('invalidly quoted string: ' + txt)
        
    
    return False


class AbstractSyntaxElement:
    
    def __str__(self):
        return repr(self)

    
    def __repr__(self):
        return '<' + self.__class__.__name__ + '>'



class Declaration(AbstractSyntaxElement):
    
    def __init__(self, name, value):
        self.name = name.strip()
        self.value = value.strip()



class TextBlock(AbstractSyntaxElement):
    
    def __init__(self, text):
        self.text = text

    
    def mergeText(self, text):
        self.text += text



class Whitespace(TextBlock):
    pass


class InsertPageBody(AbstractSyntaxElement):
    pass


class Comment(AbstractSyntaxElement):
    
    def __init__(self, text):
        self.text = text



class URLCall(AbstractSyntaxElement):
    
    def __init__(self, url):
        self.url = url



class URLForward(AbstractSyntaxElement):
    
    def __init__(self, url):
        self.url = url



class HTTPRedirect(AbstractSyntaxElement):
    
    def __init__(self, url):
        self.url = url



class Expression(AbstractSyntaxElement):
    
    def __init__(self, text):
        self.text = text.strip()



class Script(AbstractSyntaxElement, list):
    
    def __init__(self, text, lst = [], stripIndent = False):
        list.__init__(self, lst)
        self.text = text.rstrip()
        if stripIndent:
            self.text = self.stripIndent(self.text)
        

    
    def stripIndent(self, text):
        lines = text.split('\n')
        if lines:
            if not lines[0]:
                del lines[0]
            
            if not lines:
                return ''
            
            indent = len(lines[0]) - len(lines[0].lstrip())
            if indent:
                lines2 = []
                for line in lines:
                    lines2.append(line[indent:])
                    if line[:indent].strip():
                        raise ParserError('script block indentation error')
                        continue
                
                lines = lines2
            
        
        return '\n'.join(lines)

    
    def __repr__(self):
        s = [
            '<Script:']
        for c in self:
            s.append(',')
            s.append(repr(c))
        
        s.append('/Script>')
        return ''.join(s)

    
    def isStartOfBlock(self):
        if not self.text.endswith(':'):
            pass
        return sre.match('[^:#]*:[ \t]*(#.*)?$', self.text)

    
    def isEndScript(self):
        return self.text == 'end'

    
    def isEndAndNewBlock(self):
        if not sre.match('^else:', self.text):
            pass
        return sre.match('^elif\\s.+:', self.text)



class ScriptEndBlock(AbstractSyntaxElement, list):
    
    def __init__(self, content = None):
        if content is not None:
            self.append(content)
        



class Document(AbstractSyntaxElement, list):
    
    def __repr__(self):
        s = [
            '[DOCUMENT:']
        for c in self:
            s.append(',')
            s.append(repr(c))
        
        s.append(']')
        return ''.join(s)



def determineInputOutputEncodings(filename):
    inputEnc = outputEnc = None
    for line in file(filename, 'r').readlines(1000)[:10]:
        m = sre.match('.*<%@\\s*(.+)\\s*=\\s*(.+)\\s*%>\\s*$', line)
        if m:
            (name, value) = m.groups()
            name = name.lower()
            if name in ('inputencoding', 'outputencoding'):
                if stringInQuotes(value):
                    value = value[1:-1]
                
                if name == 'inputencoding':
                    inputEnc = value
                elif name == 'outputencoding':
                    outputEnc = value
                
            
        name in ('inputencoding', 'outputencoding')
    
    return (inputEnc, outputEnc)


class Parser(object):
    
    def __init__(self, stream, filename, inputEncoding):
        self.tokenizer = tokenizer.Tokenizer(stream, filename)
        self.fileLocation = os.path.dirname(filename)
        self.inputEncoding = inputEncoding
        self.tokens = self.tokenizer.getTokens()
        self.currentToken = None
        self.previousToken = None
        self.scriptNesting = 0
        self.nextToken()
        self.includedDeclarations = []

    
    def nextToken(self):
        
        try:
            self.previousToken = self.currentToken
            self.currentToken = self.tokens.next()
            return self.currentToken
        except StopIteration:
            self.currentToken = None
            return None


    
    def undoToken(self):
        (self.currentToken, self.previousToken) = (self.previousToken, None)

    
    def parse(self):
        ast = Document()
        if self.currentToken == tokenizer.TOK_COMMENTOPEN:
            self.skipComment()
        
        declarations = self.parseDeclarationBlock()
        body = self.parseDocBody()
        ast.extend(body)
        if self.currentToken is None:
            declarations.extend(self.includedDeclarations)
            declarations.reverse()
            for d in declarations:
                ast.insert(0, d)
            
            return ast
        else:
            raise ParserError('trailing garbage or wrong script tag', self.tokenizer.getLocationStr())

    
    def processWhitespaceAndComments(self):
        ast = []
        while isWhitespace(self.currentToken) or self.currentToken == tokenizer.TOK_COMMENTOPEN:
            self.skipComment()
            if isWhitespace(self.currentToken):
                ast.append(Whitespace(self.currentToken))
                self.nextToken()
                continue
            break
        return ast

    
    def skipComment(self):
        if self.currentToken == tokenizer.TOK_COMMENTOPEN:
            while self.currentToken != tokenizer.TOK_COMMENTCLOSE:
                self.nextToken()
                if not (self.currentToken):
                    raise ParserError('missing comment closing tag', self.tokenizer.getLocationStr())
                    continue
            self.nextToken()
        

    
    def parseDeclarationBlock(self):
        ast = []
        while self.currentToken == tokenizer.TOK_DECLARATIONOPEN or type(self.currentToken) in types.StringTypes:
            ast.extend(self.processWhitespaceAndComments())
            if self.currentToken == tokenizer.TOK_DECLARATIONOPEN:
                self.nextToken()
                if type(self.currentToken) in types.StringTypes:
                    (name, value) = self.currentToken.split('=', 1)
                    ast.append(Declaration(name.strip(), value.strip()))
                    if self.nextToken() == tokenizer.TOK_SCRIPTCLOSE:
                        self.nextToken()
                    else:
                        raise ParserError('close token expected', self.tokenizer.getLocationStr())
                else:
                    raise ParserError('text expected', self.tokenizer.getLocationStr())
            type(self.currentToken) in types.StringTypes
            break
        return ast

    
    def parseDocBody(self, inBlock = False):
        ast = []
        while True:
            if type(self.currentToken) in types.StringTypes:
                ast.append(TextBlock(self.currentToken))
                self.nextToken()
                continue
            if self.currentToken == tokenizer.TOK_COMMENTOPEN:
                self.skipComment()
                continue
            if self.currentToken == tokenizer.TOK_EXPRESSIONOPEN:
                ast.append(self.parseExpression())
                continue
            if self.currentToken == tokenizer.TOK_SCRIPTOPEN:
                if inBlock:
                    script = self.parseScript()
                    if script[0].isEndScript():
                        return (ast, None)
                    elif script[0].isEndAndNewBlock():
                        return (ast, script)
                    
                    ast.extend(script)
                else:
                    script = self.parseScript()
                    if len(script) == 1:
                        scripttxt = script[0].text.lower()
                        if scripttxt == 'end':
                            raise ParserError('block ended while not in block', self.tokenizer.getLocationStr())
                        
                        if scripttxt.startswith('else:') or scripttxt.startswith('elif:'):
                            raise ParserError('else or elif outside if', self.tokenizer.getLocationStr())
                        
                    
                    ast.extend(script)
            inBlock
            if self.currentToken == tokenizer.TOK_INSTRUCTIONOPEN:
                self.nextToken()
                if type(self.currentToken) in types.StringTypes:
                    if '=' in self.currentToken:
                        (instruction, value) = self.currentToken.split('=', 1)
                    else:
                        (instruction, value) = (self.currentToken, '')
                    self.processInstruction(instruction.lower(), value, ast)
                    if self.nextToken() == tokenizer.TOK_SCRIPTCLOSE:
                        self.nextToken()
                    else:
                        raise ParserError('close token expected', self.tokenizer.getLocationStr())
                else:
                    raise ParserError('text expected', self.tokenizer.getLocationStr())
            type(self.currentToken) in types.StringTypes
            if self.currentToken and type(self.currentToken) not in types.StringTypes:
                raise ParserError('normal text expected instead of tag', self.tokenizer.getLocationStr())
                continue
            break
        return ast

    
    def parseExpression(self):
        if self.currentToken == tokenizer.TOK_EXPRESSIONOPEN:
            self.nextToken()
            if type(self.currentToken) in types.StringTypes:
                expr = Expression(self.currentToken)
                self.nextToken()
                if self.currentToken == tokenizer.TOK_SCRIPTCLOSE:
                    self.nextToken()
                    return expr
                else:
                    raise ParserError('expression close expected', self.tokenizer.getLocationStr())
            else:
                raise ParserError('text expected', self.tokenizer.getLocationStr())
        else:
            raise ParserError('expression expected', self.tokenizer.getLocationStr())

    
    def parseScript(self):
        if self.currentToken == tokenizer.TOK_SCRIPTOPEN:
            self.nextToken()
            if type(self.currentToken) in types.StringTypes:
                
                try:
                    scriptNode = Script(self.currentToken, stripIndent = True)
                except ParserError:
                    px = None
                    raise ParserError(str(px), self.tokenizer.getLocationStr())

                closeTok = self.nextToken()
                if closeTok in (tokenizer.TOK_SCRIPTCLOSE, tokenizer.TOK_SCRIPTCLOSEKEEPB):
                    self.nextToken()
                    if closeTok == tokenizer.TOK_SCRIPTCLOSEKEEPB or scriptNode.isStartOfBlock():
                        newscript = self.parseScriptBlock(scriptNode)
                        if newscript is not None:
                            ast = [
                                scriptNode]
                            ast.extend(newscript)
                            return ast
                        
                    
                    return [
                        scriptNode]
                else:
                    raise ParserError('script close expected', self.tokenizer.getLocationStr())
            else:
                raise ParserError('text expected', self.tokenizer.getLocationStr())
        else:
            raise ParserError('script expected', self.tokenizer.getLocationStr())

    
    def parseScriptBlock(self, scriptNode):
        something = self.parseDocBody(inBlock = True)
        if type(something) == types.TupleType:
            (body, script) = something
            if len(body) >= 1:
                scriptNode.extend(body)
            
            return script
        else:
            raise ParserError('syntax error, script end tag missing?', self.tokenizer.getLocationStr())

    
    def processInstruction(self, instr, value, ast):
        value = value.strip()
        if instr == 'include':
            if not stringInQuotes(value):
                raise ParserError('argument for include must be a quoted string', self.tokenizer.getLocationStr())
            
            value = value[1:-1]
            filename = os.path.join(self.fileLocation, value)
            print 'parsing included file', value
            (inputenc, outputenc) = determineInputOutputEncodings(filename)
            
            try:
                includeStream = self.openIncludedFile(filename, encoding = inputenc)
            except Exception:
                x = None
                raise ParserError("error including file '%s': %s" % (value, x), self.tokenizer.getLocationStr())

            parsed = Parser(includeStream, filename, inputenc).parse()
            includeStream.close()
            for item in parsed:
                if isinstance(item, Declaration) and item.name == 'import':
                    self.includedDeclarations.append(item)
                    continue
            
            ast.append(Comment('BEGIN INCLUDE %s' % value))
            ast.extend(parsed)
            ast.append(Comment('END INCLUDE %s' % value))
        elif instr == 'call':
            ast.append(URLCall(value))
        elif instr == 'redirect':
            ast.append(URLForward(value))
        elif instr == 'httpredirect':
            ast.append(HTTPRedirect(value))
        elif instr == 'insertpagebody':
            ast.append(InsertPageBody())
        else:
            raise ParserError('unknown instruction', self.tokenizer.getLocationStr())

    
    def openIncludedFile(self, filename, encoding = None, raw = False):
        if raw:
            return file(filename, 'rb')
        elif encoding:
            return codecs.open(filename, mode = 'rb', encoding = encoding)
        else:
            return file(filename, 'r')



def syntaxTreeToStr(c):
    s = ''
    if isinstance(c, Document):
        for child in c:
            s += syntaxTreeToStr(child)
        
    elif isinstance(c, Declaration):
        s += '<%%@%s=%s@%%>' % (c.name, c.value)
    elif isinstance(c, URLCall):
        s += '\n----> CALL URL HERE: ' + c.url + '\n'
    elif isinstance(c, URLForward):
        s += '\n----> REDIRECT URL HERE: ' + c.url + '\n'
    elif isinstance(c, Comment):
        s += '\n# ' + c.text + '\n'
    elif isinstance(c, TextBlock):
        if isinstance(c, Whitespace):
            s += '[' + c.text + ']'
        else:
            s += '{' + c.text + '}'
    elif isinstance(c, Expression):
        s += '<%%=%s%%>' % c.text
    elif isinstance(c, Script):
        s += '<%SCRIPTID:' + `id(c)` + '\t' + c.text + '%>'
        for child in c:
            s += syntaxTreeToStr(child)
        
        s += '<%end SCRIPTID:' + `id(c)` + '%>'
    elif isinstance(c, InsertPageBody):
        s += '\n----> TEMPLATED PAGE BODY HERE\n'
    else:
        raise ValueError('invalid syntax ' + repr(c))
    return s


def main(args):
    if len(args) > 2:
        import codecs
        inputstream = codecs.open(args[1], mode = 'rb', encoding = args[2])
        print 'Reading with input encoding ', args[2]
        inputenc = args[2]
    else:
        inputstream = open(args[1], 'r')
        print 'Reading with default encoding'
        inputenc = None
    parser = Parser(inputstream, args[1], inputenc)
    syntaxtree = parser.parse()
    print syntaxtree
    print '*' * 70
    string = syntaxTreeToStr(syntaxtree)
    if len(args) > 2:
        print string.encode('iso-8859-1', 'xmlcharrefreplace')
    else:
        print string

if __name__ == '__main__':
    import sys
    main(sys.argv)

