# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

from __future__ import generators
import types
import weakref

class TokenizerError(Exception):
    
    def __init__(self, tokenizer, msg):
        msg = msg + ' @' + str(tokenizer.getLocationStr())
        Exception.__init__(self, msg)


UTF8_BOM = u'﻿'
TOK_DECLARATIONOPEN = 1000
TOK_EXPRESSIONOPEN = 1001
TOK_SCRIPTOPEN = 1002
TOK_SCRIPTCLOSE = 1003
TOK_SCRIPTCLOSEKEEPB = 1004
TOK_COMMENTOPEN = 1005
TOK_COMMENTCLOSE = 1006
TOK_INSTRUCTIONOPEN = 1007

class BufferedChars(object):
    
    def __init__(self, tokenizerref, stream):
        self.stream = stream
        self.chars = []
        self.nextIdx = 0
        self.tokenizerref = tokenizerref
        self.previousChar = None
        self.linenumber = 0
        x = self.next()
        if not (type(x) is types.UnicodeType) or x != UTF8_BOM:
            self.unread()
        

    
    def next(self):
        
        try:
            self.previousChar = self.chars[self.nextIdx - 1]
            self.nextIdx += 1
            return self.chars[self.nextIdx - 1]
        except IndexError:
            self.linenumber += 1
            if self.chars:
                self.previousChar = self.chars[-1]
            
            self.chars = self.stream.readline()
            if self.chars:
                self.nextIdx = 1
                return self.chars[0]
            else:
                self.nextIdx = -1
                return None
        except:
            self.chars


    
    def peek(self):
        return self.chars[self.nextIdx]

    
    def previous(self):
        return self.previousChar

    
    def unread(self):
        self.nextIdx -= 1
        if self.nextIdx < 0:
            raise TokenizerError(self.tokenizerref(), 'cannot unread char: buffer empty')
        

    
    def getLineNumber(self):
        return self.linenumber

    
    def getColumn(self):
        return self.nextIdx



class Tokenizer(object):
    
    def __init__(self, stream, filename):
        self.chars = BufferedChars(weakref.ref(self), stream)
        self.filename = filename

    
    def getLocationStr(self):
        return 'line %d col %d (file: %s)' % (self.chars.getLineNumber(), self.chars.getColumn(), self.filename)

    
    def getTokens(self):
        buffer = []
        while True:
            c = self.chars.next()
            if not c:
                break
                continue
            if c == '<' and self.chars.peek() == '%':
                if buffer:
                    yield ''.join(buffer)
                    buffer = []
                
                self.chars.next()
                next = self.chars.peek()
                if next == '@':
                    self.chars.next()
                    yield TOK_DECLARATIONOPEN
                elif next == '$':
                    self.chars.next()
                    yield TOK_INSTRUCTIONOPEN
                elif next == '=':
                    self.chars.next()
                    yield TOK_EXPRESSIONOPEN
                elif next == '!':
                    self.chars.next()
                    if self.chars.next() == '-':
                        if self.chars.next() == '-':
                            yield TOK_COMMENTOPEN
                            continue
                        
                        self.chars.unread()
                    
                    self.chars.unread()
                    yield TOK_SCRIPTOPEN
                else:
                    yield TOK_SCRIPTOPEN
            next == '@'
            if c == '-' and self.chars.peek() == '%':
                if self.chars.previous() == '-':
                    self.chars.next()
                    if self.chars.peek() == '>':
                        self.chars.next()
                        if len(buffer) > 1:
                            yield ''.join(buffer[:-1])
                        
                        buffer = []
                        yield TOK_COMMENTCLOSE
                        continue
                    else:
                        self.chars.unread()
                
                buffer.append('-')
                continue
            if c == '%' and self.chars.peek() == '>':
                if self.chars.previous() == '\\':
                    tag = TOK_SCRIPTCLOSEKEEPB
                    del buffer[-1]
                else:
                    tag = TOK_SCRIPTCLOSE
                if buffer:
                    yield ''.join(buffer)
                    buffer = []
                
                yield tag
                self.chars.next()
                while True:
                    c = self.chars.next()
                    if not c:
                        break
                    
                    if not c.isspace():
                        self.chars.unread()
                        break
                    
                    if c == '\n':
                        if self.chars.previous() == '\r':
                            buffer[-1] = '\n'
                        else:
                            buffer.append(c)
                        break
                    
                    buffer.append(c)
                if buffer:
                    yield ''.join(buffer)
                    buffer = []
                
            buffer
            if c == '\n' and self.chars.previous() == '\r':
                buffer[-1] = '\n'
                continue
            buffer.append(c)
        if buffer:
            yield ''.join(buffer)
            buffer = []
        



def main2(args):
    tok = Tokenizer(open(args[1], 'r'), args[1])
    for t in tok.getTokens():
        print `t`
    


def main(args):
    if len(args) > 2:
        import codecs
        inputstream = codecs.open(args[1], mode = 'rb', encoding = args[2])
        print 'Reading with input encoding ', args[2]
    else:
        inputstream = open(args[1], 'r')
        print 'Reading with default encoding'
    tok = Tokenizer(inputstream, args[1])
    s = ''
    for t in tok.getTokens():
        if t == TOK_DECLARATIONOPEN:
            s += '<%@'
            continue
        if t == TOK_INSTRUCTIONOPEN:
            s += '<%$'
            continue
        if t == TOK_EXPRESSIONOPEN:
            s += '<%='
            continue
        if t == TOK_SCRIPTOPEN:
            s += '<%'
            continue
        if t == TOK_SCRIPTCLOSE:
            s += '%>'
            continue
        if t == TOK_SCRIPTCLOSEKEEPB:
            s += '\\%>'
            continue
        if t == TOK_COMMENTOPEN:
            s += '<%!--'
            continue
        if t == TOK_COMMENTCLOSE:
            s += '--%>'
            continue
        if type(t) in types.StringTypes:
            s += '{' + t + '}'
            continue
        raise ValueError('invalid token ' + repr(t))
    
    print '*' * 70
    if len(args) > 2:
        print s.encode('ISO-8859-15', 'xmlcharrefreplace')
    else:
        print s

if __name__ == '__main__':
    import sys
    main(sys.argv)

