# Source Generated with Decompyle++
# File: in.pyc (Python 2.3)

'''An extensible library for opening URLs using a variety of protocols

The simplest way to use this module is to call the urlopen function,
which accepts a string containing a URL or a Request object (described
below).  It opens the URL and returns the results as file-like
object; the returned object has some extra methods described below.

The OpenerDirector manages a collection of Handler objects that do
all the actual work.  Each Handler implements a particular protocol or
option.  The OpenerDirector is a composite object that invokes the
Handlers needed to open the requested URL.  For example, the
HTTPHandler performs HTTP GET and POST requests and deals with
non-error returns.  The HTTPRedirectHandler automatically deals with
HTTP 301, 302, 303 and 307 redirect errors, and the HTTPDigestAuthHandler
deals with digest authentication.

urlopen(url, data=None) -- basic usage is that same as original
urllib.  pass the url and optionally data to post to an HTTP URL, and
get a file-like object back.  One difference is that you can also pass
a Request instance instead of URL.  Raises a URLError (subclass of
IOError); for HTTP errors, raises an HTTPError, which can also be
treated as a valid response.

build_opener -- function that creates a new OpenerDirector instance.
will install the default handlers.  accepts one or more Handlers as
arguments, either instances or Handler classes that it will
instantiate.  if one of the argument is a subclass of the default
handler, the argument will be installed instead of the default.

install_opener -- installs a new opener as the default opener.

objects of interest:
OpenerDirector --

Request -- an object that encapsulates the state of a request.  the
state can be a simple as the URL.  it can also include extra HTTP
headers, e.g. a User-Agent.

BaseHandler --

exceptions:
URLError-- a subclass of IOError, individual protocols have their own
specific subclass

HTTPError-- also a valid HTTP response, so you can treat an HTTP error
as an exceptional event or valid response

internals:
BaseHandler and parent
_call_chain conventions

Example usage:

import urllib2

# set up authentication info
authinfo = urllib2.HTTPBasicAuthHandler()
authinfo.add_password(\'realm\', \'host\', \'username\', \'password\')

proxy_support = urllib2.ProxyHandler({"http" : "http://ahad-haam:3128"})

# build a new opener that adds authentication and caching FTP handlers
opener = urllib2.build_opener(proxy_support, authinfo, urllib2.CacheFTPHandler)

# install it
urllib2.install_opener(opener)

f = urllib2.urlopen(\'http://www.python.org/\')


'''
import socket
import httplib
import inspect
import re
import base64
import urlparse
import md5
import mimetypes
import mimetools
import rfc822
import ftplib
import sys
import time
import os
import gopherlib
import posixpath

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO


try:
    import sha
except ImportError:
    sha = None

from urllib import unwrap, unquote, splittype, splithost, addinfourl, splitport, splitgophertype, splitquery, splitattr, ftpwrapper, noheaders
from urllib import getproxies
from urllib import localhost, url2pathname
__version__ = '2.0a1'
_opener = None

def urlopen(url, data = None):
    global _opener
    if _opener is None:
        _opener = build_opener()
    
    return _opener.open(url, data)


def install_opener(opener):
    global _opener
    _opener = opener


class URLError(IOError):
    
    def __init__(self, reason):
        self.reason = reason

    
    def __str__(self):
        return '<urlopen error %s>' % self.reason



class HTTPError(URLError, addinfourl):
    '''Raised when HTTP error occurs, but also acts like non-error return'''
    __super_init = addinfourl.__init__
    
    def __init__(self, url, code, msg, hdrs, fp):
        self.code = code
        self.msg = msg
        self.hdrs = hdrs
        self.fp = fp
        self.filename = url
        if fp is not None:
            self._HTTPError__super_init(fp, hdrs, url)
        

    
    def __str__(self):
        return 'HTTP Error %s: %s' % (self.code, self.msg)

    
    def __del__(self):
        if self.fp:
            self.fp.close()
        



class GopherError(URLError):
    pass


class Request:
    
    def __init__(self, url, data = None, headers = { }):
        self._Request__original = unwrap(url)
        self.type = None
        self.host = None
        self.port = None
        self.data = data
        self.headers = { }
        for key, value in headers.items():
            self.add_header(key, value)
        

    
    def __getattr__(self, attr):
        if attr[:12] == '_Request__r_':
            name = attr[12:]
            if hasattr(Request, 'get_' + name):
                getattr(self, 'get_' + name)()
                return getattr(self, attr)
            
        
        raise AttributeError, attr

    
    def get_method(self):
        if self.has_data():
            return 'POST'
        else:
            return 'GET'

    
    def add_data(self, data):
        self.data = data

    
    def has_data(self):
        return self.data is not None

    
    def get_data(self):
        return self.data

    
    def get_full_url(self):
        return self._Request__original

    
    def get_type(self):
        if self.type is None:
            (self.type, self._Request__r_type) = splittype(self._Request__original)
            if self.type is None:
                raise ValueError, 'unknown url type: %s' % self._Request__original
            
        
        return self.type

    
    def get_host(self):
        if self.host is None:
            (self.host, self._Request__r_host) = splithost(self._Request__r_type)
            if self.host:
                self.host = unquote(self.host)
            
        
        return self.host

    
    def get_selector(self):
        return self._Request__r_host

    
    def set_proxy(self, host, type):
        (self.host, self.type) = (host, type)
        self._Request__r_host = self._Request__original

    
    def add_header(self, key, val):
        self.headers[key.capitalize()] = val



class OpenerDirector:
    
    def __init__(self):
        server_version = 'Python-urllib/%s' % __version__
        self.addheaders = [
            ('User-agent', server_version)]
        self.handlers = []
        self.handle_open = { }
        self.handle_error = { }

    
    def add_handler(self, handler):
        added = 0
        for meth in dir(handler):
            if meth[-5:] == '_open':
                protocol = meth[:-5]
                if protocol in self.handle_open:
                    self.handle_open[protocol].append(handler)
                    self.handle_open[protocol].sort()
                else:
                    self.handle_open[protocol] = [
                        handler]
                added = 1
                continue
            
            i = meth.find('_')
            j = meth[i + 1:].find('_') + i + 1
            if j != -1 and meth[i + 1:j] == 'error':
                proto = meth[:i]
                kind = meth[j + 1:]
                
                try:
                    kind = int(kind)
                except ValueError:
                    pass

                dict = self.handle_error.get(proto, { })
                if kind in dict:
                    dict[kind].append(handler)
                    dict[kind].sort()
                else:
                    dict[kind] = [
                        handler]
                self.handle_error[proto] = dict
                added = 1
                continue
                continue
        
        if added:
            self.handlers.append(handler)
            self.handlers.sort()
            handler.add_parent(self)
        

    
    def __del__(self):
        self.close()

    
    def close(self):
        for handler in self.handlers:
            handler.close()
        
        self.handlers = []

    
    def _call_chain(self, chain, kind, meth_name, *args):
        handlers = chain.get(kind, ())
        for handler in handlers:
            func = getattr(handler, meth_name)
            result = func(*args)
            if result is not None:
                return result
                continue
        

    
    def open(self, fullurl, data = None):
        if isinstance(fullurl, basestring):
            req = Request(fullurl, data)
        else:
            req = fullurl
            if data is not None:
                req.add_data(data)
            
        result = self._call_chain(self.handle_open, 'default', 'default_open', req)
        if result:
            return result
        
        type_ = req.get_type()
        result = self._call_chain(self.handle_open, type_, type_ + '_open', req)
        if result:
            return result
        
        return self._call_chain(self.handle_open, 'unknown', 'unknown_open', req)

    
    def error(self, proto, *args):
        if proto in [
            'http',
            'https']:
            dict = self.handle_error['http']
            proto = args[2]
            meth_name = 'http_error_%d' % proto
            http_err = 1
            orig_args = args
        else:
            dict = self.handle_error
            meth_name = proto + '_error'
            http_err = 0
        args = (dict, proto, meth_name) + args
        result = self._call_chain(*args)
        if result:
            return result
        
        if http_err:
            args = (dict, 'default', 'http_error_default') + orig_args
            return self._call_chain(*args)
        



def build_opener(*handlers):
    '''Create an opener object from a list of handlers.

    The opener will use several default handlers, including support
    for HTTP and FTP.

    If any of the handlers passed as arguments are subclasses of the
    default handlers, the default handlers will not be used.
    '''
    opener = OpenerDirector()
    default_classes = [
        ProxyHandler,
        UnknownHandler,
        HTTPHandler,
        HTTPDefaultErrorHandler,
        HTTPRedirectHandler,
        FTPHandler,
        FileHandler]
    if hasattr(httplib, 'HTTPS'):
        default_classes.append(HTTPSHandler)
    
    skip = []
    for klass in default_classes:
        for check in handlers:
            if inspect.isclass(check):
                if issubclass(check, klass):
                    skip.append(klass)
                
            issubclass(check, klass)
            if isinstance(check, klass):
                skip.append(klass)
                continue
        
    
    for klass in skip:
        default_classes.remove(klass)
    
    for klass in default_classes:
        opener.add_handler(klass())
    
    for h in handlers:
        if inspect.isclass(h):
            h = h()
        
        opener.add_handler(h)
    
    return opener


class BaseHandler:
    handler_order = 500
    
    def add_parent(self, parent):
        self.parent = parent

    
    def close(self):
        self.parent = None

    
    def __lt__(self, other):
        if not hasattr(other, 'handler_order'):
            return True
        
        return self.handler_order < other.handler_order



class HTTPDefaultErrorHandler(BaseHandler):
    
    def http_error_default(self, req, fp, code, msg, hdrs):
        raise HTTPError(req.get_full_url(), code, msg, hdrs, fp)



class HTTPRedirectHandler(BaseHandler):
    
    def redirect_request(self, req, fp, code, msg, headers, newurl):
        """Return a Request or None in response to a redirect.

        This is called by the http_error_30x methods when a
        redirection response is received.  If a redirection should
        take place, return a new Request to allow http_error_30x to
        perform the redirect.  Otherwise, raise HTTPError if no-one
        else should try to handle this url.  Return None if you can't
        but another Handler might.
        """
        m = req.get_method()
        if code in (301, 302, 303, 307) and m in ('GET', 'HEAD') and code in (301, 302, 303) and m == 'POST':
            return Request(newurl, headers = req.headers)
        else:
            raise HTTPError(req.get_full_url(), code, msg, headers, fp)

    
    def http_error_302(self, req, fp, code, msg, headers):
        if 'location' in headers:
            newurl = headers['location']
        elif 'uri' in headers:
            newurl = headers['uri']
        else:
            return None
        newurl = urlparse.urljoin(req.get_full_url(), newurl)
        new = self.redirect_request(req, fp, code, msg, headers, newurl)
        if new is None:
            return None
        
        new.error_302_dict = { }
        if hasattr(req, 'error_302_dict'):
            if len(req.error_302_dict) > 10 or newurl in req.error_302_dict:
                raise HTTPError(req.get_full_url(), code, self.inf_msg + msg, headers, fp)
            
            new.error_302_dict.update(req.error_302_dict)
        
        new.error_302_dict[newurl] = newurl
        fp.read()
        fp.close()
        return self.parent.open(new)

    http_error_301 = http_error_303 = http_error_307 = http_error_302
    inf_msg = 'The HTTP server returned a redirect error that would lead to an infinite loop.\nThe last 30x error message was:\n'


class ProxyHandler(BaseHandler):
    handler_order = 100
    
    def __init__(self, proxies = None):
        if proxies is None:
            proxies = getproxies()
        
        if not hasattr(proxies, 'has_key'):
            raise AssertionError, 'proxies must be a mapping'
        self.proxies = proxies
        for type, url in proxies.items():
            setattr(self, '%s_open' % type, (lambda r, proxy = url, type = type, meth = self.proxy_open: meth(r, proxy, type)))
        

    
    def proxy_open(self, req, proxy, type):
        orig_type = req.get_type()
        (type, r_type) = splittype(proxy)
        (host, XXX) = splithost(r_type)
        if '@' in host:
            (user_pass, host) = host.split('@', 1)
            if ':' in user_pass:
                (user, password) = user_pass.split(':', 1)
                user_pass = base64.encodestring('%s:%s' % (unquote(user), unquote(password)))
                req.add_header('Proxy-authorization', 'Basic ' + user_pass)
            
        
        host = unquote(host)
        req.set_proxy(host, type)
        if orig_type == type:
            return None
        else:
            return self.parent.open(req)



class CustomProxy:
    
    def __init__(self, proto, func = None, proxy_addr = None):
        self.proto = proto
        self.func = func
        self.addr = proxy_addr

    
    def handle(self, req):
        if self.func and self.func(req):
            return 1
        

    
    def get_proxy(self):
        return self.addr



class CustomProxyHandler(BaseHandler):
    handler_order = 100
    
    def __init__(self, *proxies):
        self.proxies = { }

    
    def proxy_open(self, req):
        proto = req.get_type()
        
        try:
            proxies = self.proxies[proto]
        except KeyError:
            return None

        for p in proxies:
            if p.handle(req):
                req.set_proxy(p.get_proxy())
                return self.parent.open(req)
                continue
        
        return None

    
    def do_proxy(self, p, req):
        return self.parent.open(req)

    
    def add_proxy(self, cpo):
        if cpo.proto in self.proxies:
            self.proxies[cpo.proto].append(cpo)
        else:
            self.proxies[cpo.proto] = [
                cpo]



class HTTPPasswordMgr:
    
    def __init__(self):
        self.passwd = { }

    
    def add_password(self, realm, uri, user, passwd):
        if isinstance(uri, basestring):
            uri = [
                uri]
        
        uri = tuple(map(self.reduce_uri, uri))
        if not (realm in self.passwd):
            self.passwd[realm] = { }
        
        self.passwd[realm][uri] = (user, passwd)

    
    def find_user_password(self, realm, authuri):
        domains = self.passwd.get(realm, { })
        authuri = self.reduce_uri(authuri)
        for uris, authinfo in domains.iteritems():
            for uri in uris:
                if self.is_suburi(uri, authuri):
                    return authinfo
                    continue
            
        
        return (None, None)

    
    def reduce_uri(self, uri):
        '''Accept netloc or URI and extract only the netloc and path'''
        parts = urlparse.urlparse(uri)
        if parts[1]:
            if not parts[2]:
                pass
            return (parts[1], '/')
        else:
            return (parts[2], '/')

    
    def is_suburi(self, base, test):
        '''Check if test is below base in a URI tree

        Both args must be URIs in reduced form.
        '''
        if base == test:
            return True
        
        if base[0] != test[0]:
            return False
        
        common = posixpath.commonprefix((base[1], test[1]))
        if len(common) == len(base[1]):
            return True
        
        return False



class HTTPPasswordMgrWithDefaultRealm(HTTPPasswordMgr):
    
    def find_user_password(self, realm, authuri):
        (user, password) = HTTPPasswordMgr.find_user_password(self, realm, authuri)
        if user is not None:
            return (user, password)
        
        return HTTPPasswordMgr.find_user_password(self, None, authuri)



class AbstractBasicAuthHandler:
    rx = re.compile('[ \t]*([^ \t]+)[ \t]+realm="([^"]*)"', re.I)
    
    def __init__(self, password_mgr = None):
        if password_mgr is None:
            password_mgr = HTTPPasswordMgr()
        
        self.passwd = password_mgr
        self.add_password = self.passwd.add_password

    
    def http_error_auth_reqed(self, authreq, host, req, headers):
        authreq = headers.get(authreq, None)
        if authreq:
            mo = AbstractBasicAuthHandler.rx.match(authreq)
            if mo:
                (scheme, realm) = mo.groups()
                if scheme.lower() == 'basic':
                    return self.retry_http_basic_auth(host, req, realm)
                
            
        

    
    def retry_http_basic_auth(self, host, req, realm):
        (user, pw) = self.passwd.find_user_password(realm, host)
        if pw:
            raw = '%s:%s' % (user, pw)
            auth = 'Basic %s' % base64.encodestring(raw).strip()
            if req.headers.get(self.auth_header, None) == auth:
                return None
            
            req.add_header(self.auth_header, auth)
            return self.parent.open(req)
        else:
            return None



class HTTPBasicAuthHandler(AbstractBasicAuthHandler, BaseHandler):
    auth_header = 'Authorization'
    
    def http_error_401(self, req, fp, code, msg, headers):
        host = urlparse.urlparse(req.get_full_url())[1]
        return self.http_error_auth_reqed('www-authenticate', host, req, headers)



class ProxyBasicAuthHandler(AbstractBasicAuthHandler, BaseHandler):
    auth_header = 'Proxy-authorization'
    
    def http_error_407(self, req, fp, code, msg, headers):
        host = req.get_host()
        return self.http_error_auth_reqed('proxy-authenticate', host, req, headers)



class AbstractDigestAuthHandler:
    
    def __init__(self, passwd = None):
        if passwd is None:
            passwd = HTTPPasswordMgr()
        
        self.passwd = passwd
        self.add_password = self.passwd.add_password

    
    def http_error_auth_reqed(self, authreq, host, req, headers):
        authreq = headers.get(self.auth_header, None)
        if authreq:
            kind = authreq.split()[0]
            if kind == 'Digest':
                return self.retry_http_digest_auth(req, authreq)
            
        

    
    def retry_http_digest_auth(self, req, auth):
        (token, challenge) = auth.split(' ', 1)
        chal = parse_keqv_list(parse_http_list(challenge))
        auth = self.get_authorization(req, chal)
        if auth:
            auth_val = 'Digest %s' % auth
            if req.headers.get(self.auth_header, None) == auth_val:
                return None
            
            req.add_header(self.auth_header, auth_val)
            resp = self.parent.open(req)
            return resp
        

    
    def get_authorization(self, req, chal):
        
        try:
            realm = chal['realm']
            nonce = chal['nonce']
            algorithm = chal.get('algorithm', 'MD5')
            opaque = chal.get('opaque', None)
        except KeyError:
            return None

        (H, KD) = self.get_algorithm_impls(algorithm)
        if H is None:
            return None
        
        (user, pw) = self.passwd.find_user_password(realm, req.get_full_url())
        if user is None:
            return None
        
        if req.has_data():
            entdig = self.get_entity_digest(req.get_data(), chal)
        else:
            entdig = None
        A1 = '%s:%s:%s' % (user, realm, pw)
        if not req.has_data() and 'POST':
            pass
        A2 = '%s:%s' % ('GET', req.get_selector())
        respdig = KD(H(A1), '%s:%s' % (nonce, H(A2)))
        base = 'username="%s", realm="%s", nonce="%s", uri="%s", response="%s"' % (user, realm, nonce, req.get_selector(), respdig)
        if opaque:
            base = base + ', opaque="%s"' % opaque
        
        if entdig:
            base = base + ', digest="%s"' % entdig
        
        if algorithm != 'MD5':
            base = base + ', algorithm="%s"' % algorithm
        
        return base

    
    def get_algorithm_impls(self, algorithm):
        if algorithm == 'MD5':
            
            H = lambda x, e = encode_digest: e(md5.new(x).digest())
        elif algorithm == 'SHA':
            
            H = lambda x, e = encode_digest: e(sha.new(x).digest())
        
        
        KD = lambda s, d, H = H: H('%s:%s' % (s, d))
        return (H, KD)

    
    def get_entity_digest(self, data, chal):
        return None



class HTTPDigestAuthHandler(BaseHandler, AbstractDigestAuthHandler):
    '''An authentication protocol defined by RFC 2069

    Digest authentication improves on basic authentication because it
    does not transmit passwords in the clear.
    '''
    auth_header = 'Authorization'
    
    def http_error_401(self, req, fp, code, msg, headers):
        host = urlparse.urlparse(req.get_full_url())[1]
        self.http_error_auth_reqed('www-authenticate', host, req, headers)



class ProxyDigestAuthHandler(BaseHandler, AbstractDigestAuthHandler):
    auth_header = 'Proxy-Authorization'
    
    def http_error_407(self, req, fp, code, msg, headers):
        host = req.get_host()
        self.http_error_auth_reqed('proxy-authenticate', host, req, headers)



def encode_digest(digest):
    hexrep = []
    for c in digest:
        n = ord(c) >> 4 & 15
        hexrep.append(hex(n)[-1])
        n = ord(c) & 15
        hexrep.append(hex(n)[-1])
    
    return ''.join(hexrep)


class AbstractHTTPHandler(BaseHandler):
    
    def do_open(self, http_class, req):
        host = req.get_host()
        if not host:
            raise URLError('no host given')
        
        h = http_class(host)
        if req.has_data():
            data = req.get_data()
            h.putrequest('POST', req.get_selector())
            if not ('Content-type' in req.headers):
                h.putheader('Content-type', 'application/x-www-form-urlencoded')
            
            if not ('Content-length' in req.headers):
                h.putheader('Content-length', '%d' % len(data))
            
        else:
            h.putrequest('GET', req.get_selector())
        (scheme, sel) = splittype(req.get_selector())
        (sel_host, sel_path) = splithost(sel)
        if not sel_host:
            pass
        h.putheader('Host', host)
        for name, value in self.parent.addheaders:
            name = name.capitalize()
            if name not in req.headers:
                h.putheader(name, value)
                continue
        
        for k, v in req.headers.items():
            h.putheader(k, v)
        
        
        try:
            h.endheaders()
        except socket.error:
            err = None
            raise URLError(err)

        if req.has_data():
            h.send(data)
        
        (code, msg, hdrs) = h.getreply()
        fp = h.getfile()
        if code == 200:
            return addinfourl(fp, hdrs, req.get_full_url())
        else:
            return self.parent.error('http', req, fp, code, msg, hdrs)



class HTTPHandler(AbstractHTTPHandler):
    
    def http_open(self, req):
        return self.do_open(httplib.HTTP, req)


if hasattr(httplib, 'HTTPS'):
    
    class HTTPSHandler(AbstractHTTPHandler):
        
        def https_open(self, req):
            return self.do_open(httplib.HTTPS, req)




class UnknownHandler(BaseHandler):
    
    def unknown_open(self, req):
        type = req.get_type()
        raise URLError('unknown url type: %s' % type)



def parse_keqv_list(l):
    '''Parse list of key=value strings where keys are not duplicated.'''
    parsed = { }
    for elt in l:
        (k, v) = elt.split('=', 1)
        if v[0] == '"' and v[-1] == '"':
            v = v[1:-1]
        
        parsed[k] = v
    
    return parsed


def parse_http_list(s):
    '''Parse lists as described by RFC 2068 Section 2.

    In particular, parse comman-separated lists where the elements of
    the list may include quoted-strings.  A quoted-string could
    contain a comma.
    '''
    list = []
    end = len(s)
    i = 0
    inquote = 0
    start = 0
    while i < end:
        cur = s[i:]
        c = cur.find(',')
        q = cur.find('"')
        if c == -1:
            list.append(s[start:])
            break
        
        if q == -1:
            if inquote:
                raise ValueError, 'unbalanced quotes'
            else:
                list.append(s[start:i + c])
                i = i + c + 1
        
        if inquote:
            if q < c:
                list.append(s[start:i + c])
                i = i + c + 1
                start = i
                inquote = 0
            else:
                i = i + q
        q < c
        if c < q:
            list.append(s[start:i + c])
            i = i + c + 1
            start = i
            continue
        inquote = 1
        i = i + q + 1
    return map((lambda x: x.strip()), list)


class FileHandler(BaseHandler):
    
    def file_open(self, req):
        url = req.get_selector()
        if url[:2] == '//' and url[2:3] != '/':
            req.type = 'ftp'
            return self.parent.open(req)
        else:
            return self.open_local_file(req)

    names = None
    
    def get_names(self):
        if FileHandler.names is None:
            FileHandler.names = (socket.gethostbyname('localhost'), socket.gethostbyname(socket.gethostname()))
        
        return FileHandler.names

    
    def open_local_file(self, req):
        host = req.get_host()
        file = req.get_selector()
        localfile = url2pathname(file)
        stats = os.stat(localfile)
        size = stats.st_size
        modified = rfc822.formatdate(stats.st_mtime)
        mtype = mimetypes.guess_type(file)[0]
        if not mtype:
            pass
        headers = mimetools.Message(StringIO('Content-type: %s\nContent-length: %d\nLast-modified: %s\n' % ('text/plain', size, modified)))
        if host:
            (host, port) = splitport(host)
        
        if not host and not port and socket.gethostbyname(host) in self.get_names():
            return addinfourl(open(localfile, 'rb'), headers, 'file:' + file)
        
        raise URLError('file not on local host')



class FTPHandler(BaseHandler):
    
    def ftp_open(self, req):
        host = req.get_host()
        if not host:
            raise IOError, ('ftp error', 'no host given')
        
        
        try:
            host = socket.gethostbyname(host)
        except socket.error:
            msg = None
            raise URLError(msg)

        (host, port) = splitport(host)
        if port is None:
            port = ftplib.FTP_PORT
        
        (path, attrs) = splitattr(req.get_selector())
        path = unquote(path)
        dirs = path.split('/')
        (dirs, file) = (dirs[:-1], dirs[-1])
        if dirs and not dirs[0]:
            dirs = dirs[1:]
        
        user = passwd = ''
        
        try:
            fw = self.connect_ftp(user, passwd, host, port, dirs)
            if not file and 'I':
                pass
            type = 'D'
            for attr in attrs:
                (attr, value) = splitattr(attr)
                if attr.lower() == 'type' and value in ('a', 'A', 'i', 'I', 'd', 'D'):
                    type = value.upper()
                    continue
            
            (fp, retrlen) = fw.retrfile(file, type)
            headers = ''
            mtype = mimetypes.guess_type(req.get_full_url())[0]
            if mtype:
                headers += 'Content-type: %s\n' % mtype
            
            if retrlen is not None and retrlen >= 0:
                headers += 'Content-length: %d\n' % retrlen
            
            sf = StringIO(headers)
            headers = mimetools.Message(sf)
            return addinfourl(fp, headers, req.get_full_url())
        except ftplib.all_errors:
            msg = None
            raise IOError, ('ftp error', msg), sys.exc_info()[2]


    
    def connect_ftp(self, user, passwd, host, port, dirs):
        fw = ftpwrapper(user, passwd, host, port, dirs)
        return fw



class CacheFTPHandler(FTPHandler):
    
    def __init__(self):
        self.cache = { }
        self.timeout = { }
        self.soonest = 0
        self.delay = 60
        self.max_conns = 16

    
    def setTimeout(self, t):
        self.delay = t

    
    def setMaxConns(self, m):
        self.max_conns = m

    
    def connect_ftp(self, user, passwd, host, port, dirs):
        key = (user, passwd, host, port)
        if key in self.cache:
            self.timeout[key] = time.time() + self.delay
        else:
            self.cache[key] = ftpwrapper(user, passwd, host, port, dirs)
            self.timeout[key] = time.time() + self.delay
        self.check_cache()
        return self.cache[key]

    
    def check_cache(self):
        t = time.time()
        if self.soonest <= t:
            for k, v in self.timeout.items():
                if v < t:
                    self.cache[k].close()
                    del self.cache[k]
                    del self.timeout[k]
                    continue
            
        
        self.soonest = min(self.timeout.values())
        if len(self.cache) == self.max_conns:
            for k, v in self.timeout.items():
                if v == self.soonest:
                    del self.cache[k]
                    del self.timeout[k]
                    break
                    continue
            
            self.soonest = min(self.timeout.values())
        



class GopherHandler(BaseHandler):
    
    def gopher_open(self, req):
        host = req.get_host()
        if not host:
            raise GopherError('no host given')
        
        host = unquote(host)
        selector = req.get_selector()
        (type, selector) = splitgophertype(selector)
        (selector, query) = splitquery(selector)
        selector = unquote(selector)
        if query:
            query = unquote(query)
            fp = gopherlib.send_query(selector, query, host)
        else:
            fp = gopherlib.send_selector(selector, host)
        return addinfourl(fp, noheaders(), req.get_full_url())



class OpenerFactory:
    default_handlers = [
        UnknownHandler,
        HTTPHandler,
        HTTPDefaultErrorHandler,
        HTTPRedirectHandler,
        FTPHandler,
        FileHandler]
    handlers = []
    replacement_handlers = []
    
    def add_handler(self, h):
        self.handlers = self.handlers + [
            h]

    
    def replace_handler(self, h):
        pass

    
    def build_opener(self):
        opener = OpenerDirector()
        for ph in self.default_handlers:
            if inspect.isclass(ph):
                ph = ph()
            
            opener.add_handler(ph)
        


if __name__ == '__main__':
    if socket.gethostname() == 'bitdiddle':
        localhost = 'bitdiddle.cnri.reston.va.us'
    elif socket.gethostname() == 'bitdiddle.concentric.net':
        localhost = 'localhost'
    else:
        localhost = None
    urls = [
        'gopher://gopher.lib.ncsu.edu/11/library/stacks/Alex',
        'gopher://gopher.vt.edu:10010/10/33',
        'file:/etc/passwd',
        'file://nonsensename/etc/passwd',
        'ftp://www.python.org/pub/python/misc/sousa.au',
        'ftp://www.python.org/pub/tmp/blat',
        'http://www.espn.com/',
        'http://www.python.org/Spanish/Inquistion/',
        ('http://www.python.org/cgi-bin/faqw.py', 'query=pythonistas&querytype=simple&casefold=yes&req=search'),
        'http://www.python.org/',
        'ftp://gatekeeper.research.compaq.com/pub/DEC/SRC/research-reports/00README-Legal-Rules-Regs']
    cfh = CacheFTPHandler()
    cfh.setTimeout(1)
    install_opener(build_opener(cfh, GopherHandler))
    for url in urls:
        if isinstance(url, tuple):
            (url, req) = url
        else:
            req = None
        print url
        
        try:
            f = urlopen(url, req)
        except IOError:
            err = None
            print 'IOError:', err
        except socket.error:
            err = None
            print 'socket.error:', err

        buf = f.read()
        f.close()
        print 'read %d bytes' % len(buf)
        print 
        time.sleep(0.10000000000000001)
    

